@@ -1,4 +1,5 @@
-;;; $Id: dtk-speak.el,v 7.0 1997/11/13 15:32:30 raman Exp $
+;;; dtk-speak.el --- Provides Emacs Lisp interface to speech server
+;;; $Id: dtk-speak.el,v 8.0 1998/05/01 14:15:46 raman Exp $
 ;;; $Author: raman $ 
 ;;; Description:  Emacs interface to the dectalk
 ;;; Keywords: Dectalk Emacs Elisp
@@ -7,14 +8,14 @@
 ;;; LCD Archive Entry:
 ;;; emacspeak| T. V. Raman |raman@adobe.com 
 ;;; A speech interface to Emacs |
-;;; $Date: 1997/11/13 15:32:30 $ |
-;;;  $Revision: 7.0 $ | 
+;;; $Date: 1998/05/01 14:15:46 $ |
+;;;  $Revision: 8.0 $ | 
 ;;; Location undetermined
 ;;;
 
 ;;}}}
 ;;{{{  Copyright:
-;;;Copyright (C) 1995, 1996, 1997  T. V. Raman  Adobe Systems Incorporated
+;;;Copyright (C) 1995, 1996, 1997, 1998   T. V. Raman  Adobe Systems Incorporated
 ;;; Copyright (c) 1994, 1995 by Digital Equipment Corporation.
 ;;; All Rights Reserved. 
 ;;;
@@ -39,35 +40,33 @@
 
 (require 'cl)
 (declaim  (optimize  (safety 0) (speed 3)))
-(require 'dtk-macros)
 (require 'dtk-tcl)
 (eval-when (compile) (require 'emacspeak-keymap))
 
 ;;{{{  Introduction:
 
 ;;; This module defines the interface to the speech device.
-;;; It relies on the module dtk-tcl to communicate with the speech driver.
-;;; The code in this module is Dectalk specific. 
-
+;;; It relies on the module dtk-tcl to communicate with the speech server.
 ;;}}}
 ;;{{{  user customizations: 
 
 (defvar dtk-startup-hook nil
-  "List of hooks to be run after starting up the Dectalk. .
+  "List of hooks to be run after starting up the speech server. .
 Set things like speech rate, punctuation mode etc in this hook. ")
 
 (defvar dtk-tcl (or (getenv "DTK_TCL" )
                     "tcl")
-  "TCL used to run the Dectalk driver.
+  "Interpreter  used to run the speech server.
  Extended tcl --tcl-- for hardware Dectalks.
 dtk-tclsh for the software Dectalk. ")
 
 (defvar dtk-program
   (or  (getenv "DTK_PROGRAM" ) "dtk-exp")
-  "The program to use to talk to the Dectalk.
+  "The program to use to talk to the speech engine.
 At present: dtk-exp for the Dectalk Express.
 dtk-mv  for the Multivoice and older Dectalks. 
 dtk-soft (not yet released)  for the software Dectalk.
+dtk-sapi (not yet released)  for the software Dectalk on NT.
  The default is dtk-exp. ")
 
 ;;}}}
@@ -86,16 +85,16 @@ important to be interrupted.  ")
 You should not modify this variable;
 Use command  dtk-set-punctuations bound to \\[dtk-set-punctuations]. . ")
 
-(defconst dtk-drivers-alist
+(defconst dtk-servers-alist
   '(("dtk-exp" . "dtk-exp")
     ("dtk-mv" . "dtk-mv")
-    ("dtk-face" . "dtk-face")
-    ("dtk-soft" . "dtk-soft")
+    ("mbrola" . "mbrola")
+    ("dtk-sapi" . "dtk-sapi")
     )
-  "Used by completing-read when prompting for the dtk driver to use. ")
+  "Used by completing-read when prompting for the dtk server to use. ")
 
-(defconst dtk-driver-prefix "dtk-"
-  "Used by completing-read when prompting for the driver name. ")
+(defconst dtk-server-prefix "dtk-"
+  "Used by completing-read when prompting for the server name. ")
 
 ;;}}}
 ;;{{{  Mapping characters to speech:
@@ -247,7 +246,7 @@ pronouncing that character.")
 (aset  table 127  "backspace")
 ;;; Characters with the 8th bit set:
 (aset  table 128  " octal 200 ")
-(aset  table 129  " octal 201 ")
+(aset  table 129  " ");shows up on WWW pages
 (aset  table 130  " octal 202 ")
 (aset  table 131  " octal 203 ")
 (aset  table 132  " octal 204 ")
@@ -493,38 +492,45 @@ pronouncing that character.")
   (aref dtk-character-to-speech-table char )))
 
 ;;}}}
-;;{{{  interactively selecting the driver:
+;;{{{  interactively selecting the server:
 
-(defun dtk-select-driver ()
-  "Select a dectalk driver interactively.
-This will be the driver that is used when you next call either
+(defun dtk-select-server (program)
+  "Select a speech server interactively.
+This will be the server that is used when you next call either
 \\[dtk-initialize] or \\[dtk-emergency-restart]. "
-  (interactive)
-  (declare (special  dtk-tcl dtk-program dtk-drivers-alist dtk-driver-prefix ))
-  (setq dtk-program
-        (completing-read "Select Dectalk driver:"
-                         dtk-drivers-alist  nil
-                         t dtk-driver-prefix ))
+  (interactive
+   (list (completing-read "Select speech server:"
+                          dtk-servers-alist  nil
+                          t dtk-server-prefix )))
+  (declare (special  dtk-tcl dtk-program dtk-servers-alist
+                     dtk-server-prefix
+                     dtk-clean-text-function))
+  (setq dtk-program program)
   (cond
    ((string= dtk-program "dtk-soft")
-      (setq dtk-tcl "dtk-tclsh"))
+    (setq dtk-tcl "dtk-tclsh"))
+   ((string= dtk-program "dtk-sapi")
+    (setq dtk-tcl 
+	  (concat exec-directory  "speak.exe"))
+    (setq dtk-clean-text-function 'dtk-clean-text))
    ((string= dtk-program "dtk-face")
-      (setq dtk-tcl "DECface"))
-    (t (setq dtk-tcl "tcl")))
-  (message "Selected %s Use %s to start using it "
-           dtk-program
-           (substitute-command-keys "\\[dtk-emergency-restart]")))
+    (setq dtk-tcl "DECface"))
+   (t (setq dtk-tcl "tcl")))
+  (when (interactive-p)
+    (message "Selected %s Use %s to start using it "
+             dtk-program
+             (substitute-command-keys "\\[dtk-emergency-restart]"))))
 
 ;;}}}
 ;;{{{  initialize the speech process
 (defvar dtk-debug nil
-  "Set this to t if you want to debug the synthesizer driver. ")
+  "Set this to t if you want to debug the synthesizer server. ")
 
-(defvar dtk-speak-driver-initialized nil
-  "Records if the driver is initialized.")
+(defvar dtk-speak-server-initialized nil
+  "Records if the server is initialized.")
 
 (defvar dtk-speak-process-connection-type nil
-  "*Specifies if we use ptys or pipes to connect to the speech driver process.
+  "*Specifies if we use ptys or pipes to connect to the speech server process.
 Has the same semantics as the builtin process-connection-type.
 Default is to use pipes.")
 
@@ -533,32 +539,38 @@ Default is to use pipes.")
   (declare (special dtk-program dtk-tcl
                     dtk-speak-process-connection-type
                     dtk-speaker-process  dtk-debug
-                    dtk-speak-driver-initialized
+                    dtk-speak-server-initialized
                     dtk-startup-hook emacspeak-dir))
     ;;; first load the appropriate interface library:
   (require 'dtk-tcl)
   (when (and dtk-speaker-process
              (or (eq 'run (process-status dtk-speaker-process ))
+                 (eq 'open (process-status dtk-speaker-process ))
                  (eq 'stop (process-status dtk-speaker-process ))))
-    (kill-process dtk-speaker-process ))
+    (delete-process dtk-speaker-process ))
+  (when (eq window-system 'win32)
+    (dtk-select-server "dtk-sapi"))
   (let ((process-connection-type  dtk-speak-process-connection-type))  
     (setq dtk-speaker-process
-          (start-process "speaker"
-                         (and dtk-debug " *speaker*")
-                         dtk-tcl
-                         (concat emacspeak-dir "/"
-                                 dtk-program  )))
-(process-kill-without-query dtk-speaker-process)
-    (setq dtk-speak-driver-initialized
-          (eq 'run (process-status dtk-speaker-process ))))
-  (unless dtk-debug 
-    (set-process-filter dtk-speaker-process 'dtk-filter-function ))
+          (apply 
+	   'start-process "speaker"
+	   (and dtk-debug " *speaker*")
+	   dtk-tcl
+	   (unless (string= dtk-program "dtk-sapi")                         
+	     (list (concat emacspeak-dir "/"
+		     dtk-program)))))
+    (process-kill-without-query dtk-speaker-process)
+    (setq dtk-speak-server-initialized
+          (or (eq 'run (process-status dtk-speaker-process ))
+              (eq 'open (process-status dtk-speaker-process )))))
+					;(unless dtk-debug 
+                                        ;(set-process-filter dtk-speaker-process 'dtk-filter-function ))
   (run-hooks 'dtk-startup-hook )
-  (unless dtk-speak-driver-initialized
-    (message "The speech driver is not running")))
+  (unless dtk-speak-server-initialized
+    (message "The speech server is not running")))
 
 (defun dtk-emergency-restart ()
-  "Use this to nuke the currently running dtk driver and restart it.
+  "Use this to nuke the currently running dtk server and restart it.
 Useful if you want to switch to another synthesizer while emacspeak is
 running.  Also useful for emergency stopping of speech. "
   (interactive)
@@ -576,14 +588,14 @@ Note: *speaker* is a hidden buffer, ie it has a leading space in its name."
   (cond
    (flag (setq dtk-debug t ))
    (t (setq dtk-debug (not dtk-debug ))))
-  (message "Turned %s debugging of the synthesizer driver %s"
+  (message "Turned %s debugging of the synthesizer server %s"
            (if dtk-debug "on" "off")
            (if dtk-debug
                (substitute-command-keys
-                "Restart the synthesizer driver  by pressing
+                "Restart the synthesizer server  by pressing
 \\[dtk-emergency-restart] to start debugging")
              (substitute-command-keys
-              "Restart the synthesizer driver  by pressing
+              "Restart the synthesizer server  by pressing
 \\[dtk-emergency-restart] to stop  debugging"))))
 
 ;;}}}
