@@ -1,4 +1,5 @@
-;;; $Id: dtk-tcl.el,v 7.0 1997/11/13 15:32:30 raman Exp $
+;;; dtk-tcl.el --- Interface to TCL speech server --uses module dtk-interp.el
+;;; $Id: dtk-tcl.el,v 8.0 1998/05/01 14:15:46 raman Exp $
 ;;; $Author: raman $ 
 ;;; Description:  Interfacing to the Dectalk via TCL. 
 ;;; Keywords: Dectalk, TCL
@@ -7,14 +8,14 @@
 ;;; LCD Archive Entry:
 ;;; emacspeak| T. V. Raman |raman@adobe.com 
 ;;; A speech interface to Emacs |
-;;; $Date: 1997/11/13 15:32:30 $ |
-;;;  $Revision: 7.0 $ | 
+;;; $Date: 1998/05/01 14:15:46 $ |
+;;;  $Revision: 8.0 $ | 
 ;;; Location undetermined
 ;;;
 
 ;;}}}
 ;;{{{  Copyright:
-;;;Copyright (C) 1995, 1996, 1997  T. V. Raman  Adobe Systems Incorporated
+;;;Copyright (C) 1995, 1996, 1997, 1998   T. V. Raman  Adobe Systems Incorporated
 ;;; Copyright (c) 1994, 1995 by Digital Equipment Corporation.
 ;;; All Rights Reserved. 
 ;;;
@@ -39,7 +40,7 @@
 
 (require 'cl)
 (declaim  (optimize  (safety 0) (speed 3)))
-(require 'dtk-macros)
+(require 'dtk-interp)
 (require 'dtk-voices)
 (require 'emacspeak-sounds)
 (require 'emacspeak-pronounce)
@@ -50,7 +51,7 @@
 ;;{{{  Introduction:
 
 ;;; This module defines the interface to the
-;;; various speech drivers written in tcl
+;;; various speech servers written in tcl
 ;;; This module is TCL and Dectalk specific. 
 
 ;;}}}
@@ -74,13 +75,86 @@ Do not set this variable by hand, use command  dtk-toggle-split-caps
 (defvar dtk-cleanup-patterns
   (list
    "." "_" "-"  "=" "/"  "+" "*" ":" ";" "%" 
-   "~" "$" ")" "#" "/\\" "<>" )
+   "{" "}" "~" "$" ")" "#" "/\\" "<>" )
   "List of repeating patterns to clean up.
 Use command  dtk-add-cleanup-pattern
  bound to \\[dtk-add-cleanup-pattern]  to add more patterns. 
 Specify patterns that people use to decorate their ASCII files, and cause
 untold pain to the speech synthesizer. ") 
 
+;;}}}
+;;{{{ cleaning up text
+
+;;;this is an alternative to doing the cleanup in the speech server.
+(defvar dtk-clean-text-function nil
+  "Set this to a function if you want to perform cleanups on the text
+before shipping it to the speech server.")
+
+
+(defsubst dtk-clean-text ()
+  (declare (special  dtk-split-caps dtk-punctuation-mode))
+  (let ((case-fold-search nil))
+    (goto-char (point-min))
+    (cond
+     ((string= dtk-punctuation-mode "all")
+  ;;; change pronunciation for # and  * and space out the rest 
+      (while
+	  (re-search-forward "[%&:;()$+=/*#]" nil t)
+	(case (preceding-char)
+	  (?# (replace-match "  pound "))
+	  (?* (replace-match " star "))
+	  (otherwise  (replace-match " \\& " t))))
+      (goto-char (point-min))
+      (while (search-forward ".," nil t)
+	(replace-match " dot comma " t))
+      (goto-char (point-min))
+      (while (search-forward "..." nil t)
+	(replace-match " dot dot dot "))
+      (goto-char (point-min))
+      (while (search-forward ".." nil t )
+	(replace-match " dot dot "))
+      (goto-char (point-min))
+      (while
+	  (re-search-forward "\\([a-zA-Z]\\)\\.\\([a-zA-Z]\\)" nil t)
+	(replace-match "\\1 dot \\2" t ))
+      (goto-char (point-min))
+      (while (re-search-forward "[0-9]+" nil t)
+	(replace-match " \\& "))
+      (goto-char (point-min)))
+     (t (goto-char (point-min))
+	(while (search-forward ".," nil t)
+	  (replace-match " "))
+	(goto-char (point-min))
+	(while (re-search-forward       
+		"\\([0-9a-zA-Z]\\)\\([\"!;/:()=]\\)+\\([0-9a-zA-z]\\)"
+		nil t)
+	  (replace-match "\\1 \\2 \\3"))
+	(goto-char (point-min))
+	(while
+	    (re-search-forward
+	     "\\([a-zA-Z]\\)\\(,\\)+\\([a-zA-z]\\)" nil t)  \
+	     (replace-match "\\1 \\2 \\3"))
+     (goto-char (point-min))
+     (while (re-search-forward
+	     "\\([a-zA-Z]\\)\\.\\([a-zA-z]\\)" nil t)
+       (replace-match "\\1 dot \\2" ))
+     (goto-char (point-min))))
+  (when dtk-split-caps
+   ;;; first space out runs of upper case
+    (goto-char (point-min))
+    (while
+	(re-search-forward "\\([A-Z]\\)\\([A-Z]\\)"  nil t)
+      (replace-match "\\1[*]\\2" t)
+      (backward-char 1))
+    (goto-char (point-min))
+    (while (re-search-forward 
+	    "\\([^ -_A-Z]\\)\\([A-Z][a-zA-Z]* \\)" nil t)
+      (replace-match "\\1[*]\\2" t))
+    (goto-char (point-min))
+    (while (re-search-forward 
+	    "\\([^ -_A-Z]\\)\\([A-Z]\\)" nil t) 
+      (replace-match "\\1[*]\\2" t)))))
+
 ;;}}}
 ;;{{{  internal variables
 
@@ -117,7 +191,7 @@ dtk-toggle-allcaps-beep bound to \\[dtk-toggle-allcaps-beep]. ")
   "Variable holding last output")
 
 (defvar dtk-speech-rate 425
-  "Rate at which dectalk talks.
+  "Rate at which tts talks.
 Do not modify this variable directly; use command  dtk-set-rate
  bound to \\[dtk-set-rate]. ")
 ;;;declared here to help compilation
@@ -165,24 +239,20 @@ Do not modify this variable directly; use command  dtk-set-rate
 (defsubst dtk-silence (duration &optional force)
   "Produce silence"
   (declare (special dtk-quiet dtk-speaker-process
-                    dtk-speak-driver-initialized))
+                    dtk-speak-server-initialized))
   (unless dtk-quiet 
-    (when dtk-speak-driver-initialized
-      (process-send-string dtk-speaker-process
-                           (format "sh %d%s\n"
-                                   duration
-                                   (if force ";d" ""))))))
+    (when dtk-speak-server-initialized
+      (dtk-interp-silence duration
+                          (if force "\nd" "")))))
 
 (defsubst dtk-tone (pitch duration &optional force)
   "Produce a tone."
   (declare (special dtk-quiet dtk-speaker-process
-                    dtk-speak-driver-initialized))
+                    dtk-speak-server-initialized))
   (unless dtk-quiet 
-    (when dtk-speak-driver-initialized
-         (process-send-string dtk-speaker-process
-                         (format "t %d %d%s\n"
-pitch duration
-(if force ";d" ""))))))
+    (when dtk-speak-server-initialized
+      (dtk-interp-tone pitch duration
+                       (if force "\nd" "")))))
 
 ;;; helper function:
 
@@ -190,12 +260,13 @@ pitch duration
 ;;; Fix brackets by changing to text.
  ;;; This is necessary because
 ;;;  [] marks dtk commands; {} is special to tcl
-
-(defconst dtk-bracket-regexp "[][{}<>|]"
-  "Brackets that are special to dtk and tcl.
+;;; Optionally post-process the text with cleanup function if one is specified.
+(defconst dtk-bracket-regexp "[][{}<>\\|]"
+  "Brackets and other chars  that are special to dtk and tcl.
  {} is special to tcl.
 [] is special to both dtk and tcl.
-<> and | are fixed to improve pronunciation.")
+<> and | are fixed to improve pronunciation.
+\\ is fixed because it tends to be a metacharacter")
 
 (defsubst  dtk-fix-brackets (mode)
   (declare  (special dtk-bracket-regexp ))
@@ -203,32 +274,37 @@ pitch duration
     (save-match-data 
       (cond
        ((string=  "all"  mode )
-        (while (re-search-forward dtk-bracket-regexp  nil t )
-          (cond
-           ((= ?| (char-after (match-beginning 0 )))
-            (replace-match " pipe "))
-           ((= ?< (char-after (match-beginning 0 )))
-            (replace-match " less than "))
-           ((= ?> (char-after (match-beginning 0 )))
-            (replace-match " greater than "))
-           ((= ?{ (char-after (match-beginning 0 )))
-            (replace-match " left brace "))
-           ((= ?} (char-after (match-beginning 0 )))
-            (replace-match " right brace "))
-           ((=  ?\] (char-after (match-beginning 0)))
-            (replace-match " right bracket "))
-           ((= ?\[ (char-after  (match-beginning 0)))
-            (replace-match " left bracket ")))))
+        (let ((start nil)
+              (personality nil))        
+          (while (re-search-forward dtk-bracket-regexp  nil t )
+            (setq start (1- (point)))
+            (setq personality 
+                  (get-text-property
+  start 'personality))
+            (cond
+             ((= ?| (char-after (match-beginning 0 )))
+              (replace-match " pipe "))
+             ((= ?< (char-after (match-beginning 0 )))
+              (replace-match " less than "))
+             ((= ?> (char-after (match-beginning 0 )))
+              (replace-match " greater than "))
+             ((= ?{ (char-after (match-beginning 0 )))
+              (replace-match " left brace "))
+             ((= ?} (char-after (match-beginning 0 )))
+              (replace-match " right brace "))
+             ((=  ?\] (char-after (match-beginning 0)))
+              (replace-match " right bracket "))
+             ((= ?\[ (char-after  (match-beginning 0)))
+              (replace-match " left bracket "))
+             ((= ?\\ (char-after (match-beginning 0 )))
+              (replace-match " backslash ")))
+            (when personality 
+              (put-text-property start (point)
+                                 'personality personality)))))
        (t (save-match-data
             (while (re-search-forward dtk-bracket-regexp   nil t )
               (replace-match " " nil t ))))))))
 
-(defsubst dtk-handle-single-caps ()
-  (goto-char (point-min))
-  (save-match-data
-    (while (re-search-forward " [A-Z] " nil t)
-    (replace-match  " cap \\&" ))))
-
 (defvar dtk-speak-nonprinting-chars t
   "*Non nil value means non printing characters  should be
 spoken as their octal value.
@@ -253,14 +329,6 @@ it seems some accented characters in certain contexts.")
            (replace-match
             (format " %s " (aref  dtk-character-to-speech-table char)))))))))
 
-
-;;; replace ".," with ". ," so we hear it.
-
-(defsubst dtk-fix-dot-comma ()
-  (goto-char (point-min))
-  (while (search-forward ".," nil t)
-    (replace-match ". ,")))
-
 (defsubst  dtk-quote(mode ) 
   (declare (special dtk-cleanup-patterns))
   (goto-char (point-min))
@@ -273,7 +341,9 @@ it seems some accented characters in certain contexts.")
     ;;; dtk will think it's processing a command otherwise:
   (dtk-fix-brackets mode)
   ;;; fix control chars
-  (dtk-fix-control-chars))
+  (dtk-fix-control-chars)
+  (and dtk-clean-text-function
+       (funcall dtk-clean-text-function)))
 
 
 (defsubst dtk-fix-backslash ()
@@ -340,11 +410,11 @@ it seems some accented characters in certain contexts.")
 (defsubst dtk-speak-using-voice (voice text)
   (declare (special dtk-voice-reset-code))
   (unless (eq 'inaudible voice )
-            (process-send-string dtk-speaker-process
-             (format "q {%s%s%s }\n"
-                     (dtk-get-voice-command voice )
-                     text
-                     dtk-voice-reset-code))))
+    (dtk-interp-queue 
+     (format "%s%s %s \n"
+             (dtk-get-voice-command voice )
+             text
+             dtk-voice-reset-code))))
 
                                         ;Internal function used by dtk-speak to send text out.
                                         ;Handles voice locking etc.
@@ -370,41 +440,34 @@ it seems some accented characters in certain contexts.")
         (if personality 
             (dtk-speak-using-voice personality 
                                    (buffer-substring start last ))
-          (process-send-string dtk-speaker-process
-                               (format "q {%s }\n"
-                                       (buffer-substring  start last))))
+          (dtk-interp-queue (buffer-substring  start last)))
         (setq start  last
               personality (get-text-property last  'personality))) ; end while
       ))                                ; end clause 
-   (t (process-send-string dtk-speaker-process
-                           (format "q {%s }\n"
-                                   (buffer-substring start end  ))))))
+   (t (dtk-interp-queue (buffer-substring start end  )))))
 
                                         ;Force the speech. 
 (defsubst dtk-force ()
   (declare (special dtk-speaker-process))
-       (process-send-string dtk-speaker-process
-                            "d\n"))
+  (dtk-interp-speak))
 
-                                        ;Write out the string to the dectalk via TCLSH.
+                                        ;Write out the string to the tts via TCLSH.
                                         ;No quoting is done, if want to quote the text, see dtk-speak
 (defsubst dtk-dispatch (string)
   (declare (special dtk-speaker-process
-                    dtk-speak-driver-initialized
+                    dtk-speak-server-initialized
                     dtk-quiet)) 
   (unless dtk-quiet
-    (when dtk-speak-driver-initialized
-         (process-send-string dtk-speaker-process
-                              (format  "dectalk_say { %s}\n" string )))))
+    (when dtk-speak-server-initialized
+      (dtk-interp-dispatch string ))))
 
 (defsubst dtk-stop ()
   "Stop speech now. "
   (interactive)
   (declare (special dtk-speaker-process
-                    dtk-speak-driver-initialized))
-  (when dtk-speak-driver-initialized
-       (process-send-string dtk-speaker-process
-                            (format "s\n" ))))
+                    dtk-speak-server-initialized))
+  (when dtk-speak-server-initialized
+    (dtk-interp-stop)))
 
 
 (defsubst dtk-reset-default-voice()
@@ -412,7 +475,6 @@ it seems some accented characters in certain contexts.")
   (dtk-dispatch (dtk-get-voice-command dtk-default-voice )))
 
 ;;}}}
-
 ;;{{{  adding cleanup patterns:
 
 (defun dtk-add-cleanup-pattern (&optional delete ) 
@@ -459,33 +521,41 @@ will say ``aw fifteen dot'' when speaking the string
                "\\(" pattern  "\\)+")))
     (save-excursion
       (save-match-data
-        (while (re-search-forward reg nil t)
-          (replace-match
-           (if  (string= "all" mode) 
-               (format " aw %s %s"
-                       (/ (- (match-end 0 ) (match-beginning 0))
-                          (length string))
-                       (if (string= " " pattern)
-                           " space " string) )
-             "")))))))
+        (let ((start nil)
+              (personality nil)
+              (replacement nil))
+          (while (re-search-forward reg nil t)
+            (setq personality 
+                  (get-text-property (point) 'personality))
+            (setq replacement
+                  (if  (string= "all" mode) 
+                      (format " aw %s %s"
+                              (/ (- (match-end 0 ) (match-beginning 0))
+                                 (length string))
+                              (if (string= " " pattern)
+                                  " space " string) )
+                    ""))
+            (replace-match replacement)
+            (setq start (- (point) (length replacement)))
+            (when personality 
+              (put-text-property start (point)
+                                 'personality personality ))))))))
 
 ;;; Uses the syntax table belonging to the buffer that owns the text
 ;;; to parse and speak the text intelligently.
 
-(defvar dtk-speak-block-and-synchronize nil
-  "If t, then dtk-speak behaves  like when called
-with wait set to t")
-
 (defvar dtk-speak-treat-embedded-punctuations-specially t
   "*If T then speech not split at embedded punctuations.")
 
+(defvar dtk-speak-skim-scale 1.2
+"*Scale factor applied to speech rate when skimming. ")
 
-(defun dtk-speak (text &optional wait)
-  "Speak the text string on the  dectalk, 
+(defun dtk-speak (text &optional skim)
+  "Speak the text string on the  tts, 
 by passing through an appropriate tcl.
 No-op if variable dtk-quiet is set to nil.
-If optional arg wait is non-nil, then this function returns after the speech
-has been completed.
+If optional arg skim is T then we skim i.e. speed up speech
+as we go along.
 If outline-minor-mode is on and selective display is in effect,
 only speak upto the first ctrl-m. "
   (declare (special dtk-speaker-process dtk-stop-immediately
@@ -493,14 +563,16 @@ only speak upto the first ctrl-m. "
                     dtk-speak-treat-embedded-punctuations-specially
                     dtk-quiet  dtk-chunk-separator-syntax
                     voice-lock-mode   dtk-punctuation-mode
+                    dtk-split-caps
                     emacspeak-pronounce-pronunciation-table
                     selective-display ))
                                         ; If you dont want me to talk,
                                         ; I wont.
   (unless  dtk-quiet
                                         ; ensure  the process  is live
-    (or (eq 'run (process-status dtk-speaker-process ))
-        (dtk-initialize) )
+    (unless (or (eq 'run (process-status dtk-speaker-process ))
+                (eq 'open (process-status dtk-speaker-process )))
+      (dtk-initialize) )
                                         ; flush previous speech if asked to 
     (when dtk-stop-immediately (dtk-stop ))
     (or (stringp text) (setq text (format "%s" text )))
@@ -520,15 +592,17 @@ only speak upto the first ctrl-m. "
           (start 1)
           (end nil )
           (mode dtk-punctuation-mode)
+          (split-caps dtk-split-caps)
           (voice-lock voice-lock-mode ))
       (save-excursion
         (set-buffer dtk-scratch-buffer )
         (let ((inhibit-read-only t))
           (erase-buffer)
                                         ; inherit environment
-          (setq dtk-chunk-separator-syntax
-                inherit-chunk-separator-syntax
+          (setq dtk-chunk-separator-syntax inherit-chunk-separator-syntax
                 emacspeak-use-auditory-icons use-auditory-icons
+                dtk-punctuation-mode mode
+                dtk-split-caps split-caps
                 dtk-speak-nonprinting-chars inherit-speak-nonprinting-chars)
           (set-syntax-table syntax-table )
           (setq voice-lock-mode voice-lock)
@@ -547,18 +621,21 @@ only speak upto the first ctrl-m. "
                                         ;treat this as a chunk only if the punctuation is followed
                                         ;by white space
           (unless (and dtk-speak-treat-embedded-punctuations-specially
-                   (char-after  (point)) 
+                       (char-after  (point)) 
                        (= (char-syntax (preceding-char )) ?.)
                        (not (= 32 (char-syntax (following-char )))))
             (setq end (point ))
             (dtk-format-text-and-speak  start end )
+            (when skim 
+              (dtk-interp-queue-set-rate
+               (round  (* dtk-speech-rate 
+                          dtk-speak-skim-scale)))
+              (setq skim nil))
             (setq start  end)))         ; end while
                                         ; process trailing text
         (or  (= start (point-max))
              (dtk-format-text-and-speak start (point-max)))))
-    (when wait (accept-process-output dtk-speaker-process ))
-    (dtk-force)
-    (when wait (accept-process-output dtk-speaker-process ))))
+    (dtk-force)))
 
 (defun dtk-speak-list (text )
   "Speak a  list of strings. "
@@ -581,49 +658,45 @@ only speak upto the first ctrl-m. "
 (defsubst dtk-letter (letter)
   "Speak a letter"
   (declare (special dtk-speaker-process
-                    dtk-speak-driver-initialized
+                    dtk-speak-server-initialized
                     dtk-quiet ))
   (unless dtk-quiet
-          (when dtk-speak-driver-initialized
-               (process-send-string dtk-speaker-process
-                       (format "l {%s}\n" letter )))))
+    (when dtk-speak-server-initialized
+      (dtk-interp-letter  letter ))))
 
                                         ;Say these words, ie speak each of them as a word.
                                         ; Text is sent via the appropriate TCLSH. 
 (defun dtk-say (words)
   "Say these words"
   (declare (special dtk-speaker-process dtk-stop-immediately
-                    dtk-speak-driver-initialized dtk-quiet))
+                    dtk-speak-server-initialized dtk-quiet))
   ;; I wont talk if you dont want me to
   (unless dtk-quiet 
     (or (eq 'run (process-status dtk-speaker-process ))
+        (eq 'open (process-status dtk-speaker-process ))
         (dtk-initialize))
-    (when dtk-speak-driver-initialized
+    (when dtk-speak-server-initialized
       (and dtk-stop-immediately (dtk-stop ))
-      (process-send-string dtk-speaker-process
-                           (format "dectalk_say {%s}\n"
-                                    words)))))
+      (dtk-interp-say words))))
 
 ;;}}}
 ;;{{{  sending commands 
 
 (defun dtk-set-rate (rate    &optional prefix)
-  "Set speaking rate for the dectalk.
+  "Set speaking rate for the tts.
 Interactive prefix arg means set   the global default value, and then set the
 current local  value to the result. "
   (interactive "nEnter new rate:\nP")
   (declare (special dtk-speech-rate dtk-speaker-process
-                    dtk-speak-driver-initialized))
-  (when dtk-speak-driver-initialized
+                    dtk-speak-server-initialized))
+  (when dtk-speak-server-initialized
     (cond
      (prefix
       (setq-default dtk-speech-rate rate )
       (setq dtk-speech-rate rate))
      (t (make-local-variable 'dtk-speech-rate )
         (setq dtk-speech-rate rate)))
-    (process-send-string dtk-speaker-process
-                         (format "dectalk_set_speech_rate %s\n"
-                                 rate))
+                         (dtk-interp-set-rate rate)
                          (message "Set speech rate to %s %s\n"
                                   rate
                                   (if prefix "" "locally"))))
@@ -665,17 +738,15 @@ Interactive prefix arg means set   the global default value, and then set the
 current local  value to the result. "
   (interactive "nEnter new factor:\nP")
   (declare (special dtk-character-scale dtk-speaker-process
-                    dtk-speak-driver-initialized))
-  (when dtk-speak-driver-initialized
+                    dtk-speak-server-initialized))
+  (when dtk-speak-server-initialized
     (cond
      (prefix
       (setq-default dtk-character-scale factor)
       (setq dtk-character-scale factor))
      (t (make-local-variable 'dtk-character-scale)
         (setq dtk-character-scale factor)))
-    (process-send-string dtk-speaker-process
-                         (format "dectalk_set_character_scale %s\n"
-                                 dtk-character-scale))
+    (dtk-interp-set-character-scale dtk-character-scale)
     (message "Set character scale factor to %s %s"
              dtk-character-scale
              (if  prefix ""  "locally"))))
@@ -727,22 +798,20 @@ means toggle the global default value, and then set the current local
 value to the result. "
   (interactive "P")
   (declare (special dtk-speaker-process dtk-split-caps
-                    dtk-speak-driver-initialized))
-  (when dtk-speak-driver-initialized
+                    dtk-speak-server-initialized))
+  (when dtk-speak-server-initialized
     (cond
-   (prefix
-    (setq-default  dtk-split-caps
-                   (not  (default-value 'dtk-split-caps )))
-    (setq dtk-split-caps (default-value 'dtk-split-caps )))
-   (t (make-local-variable 'dtk-split-caps)
-    (setq dtk-split-caps 
-            (not dtk-split-caps ))))
-  (process-send-string dtk-speaker-process
-                       (format "dectalk_split_caps %s\n"
-                               (if dtk-split-caps 1 0 )))
-  (message "Turned %s split caps mode%s "
-           (if dtk-split-caps "on" "off" )
-           (if prefix "" " locally"))))
+     (prefix
+      (setq-default  dtk-split-caps
+                     (not  (default-value 'dtk-split-caps )))
+      (setq dtk-split-caps (default-value 'dtk-split-caps )))
+     (t (make-local-variable 'dtk-split-caps)
+        (setq dtk-split-caps 
+              (not dtk-split-caps ))))
+    (dtk-interp-toggle-split-caps dtk-split-caps )
+    (message "Turned %s split caps mode%s "
+             (if dtk-split-caps "on" "off" )
+             (if prefix "" " locally"))))
 
 (defun dtk-toggle-capitalization  (&optional prefix)
   "Toggle capitalization.  when set, capitalization is indicated by a
@@ -750,8 +819,8 @@ short beep.  Interactive prefix arg means toggle the global default
 value, and then set the current local value to the result. "
   (interactive "P")
   (declare (special dtk-speaker-process dtk-capitalize
-                    dtk-speak-driver-initialized))
-  (when dtk-speak-driver-initialized
+                    dtk-speak-server-initialized))
+  (when dtk-speak-server-initialized
     (cond
      (prefix
       (setq-default  dtk-capitalize
@@ -760,9 +829,7 @@ value, and then set the current local value to the result. "
      (t (make-local-variable 'dtk-capitalize)
         (setq dtk-capitalize 
               (not dtk-capitalize ))))
-    (process-send-string dtk-speaker-process 
-                         (format "dectalk_capitalize  %s\n"
-                                 (if dtk-capitalize  1 0 )))
+    (dtk-interp-toggle-capitalization dtk-capitalize  )
     (message "Turned %s capitalization  mode%s "
              (if dtk-capitalize  "on" "off" )
              (if prefix "" " locally"))))
@@ -776,8 +843,8 @@ Note that allcaps-beep is a very useful thing when programming.
 However it is irritating to have it on when reading documents."
   (interactive "P")
   (declare (special dtk-speaker-process dtk-allcaps-beep
-                    dtk-speak-driver-initialized))
-  (when dtk-speak-driver-initialized
+                    dtk-speak-server-initialized))
+  (when dtk-speak-server-initialized
     (cond
      (prefix
       (setq-default  dtk-allcaps-beep
@@ -786,9 +853,7 @@ However it is irritating to have it on when reading documents."
      (t (make-local-variable 'dtk-allcaps-beep)
         (setq dtk-allcaps-beep 
               (not dtk-allcaps-beep ))))
-    (process-send-string dtk-speaker-process 
-                         (format "dectalk_allcaps_beep  %s\n"
-                                 (if dtk-allcaps-beep  1 0 )))
+    (dtk-interp-toggle-allcaps-beep dtk-allcaps-beep  )
     (message "Turned %s allcaps-beep  mode%s "
              (if dtk-allcaps-beep  "on" "off" )
              (if prefix "" " locally"))))
@@ -806,18 +871,16 @@ current local  value to the result. "
                       t)
     current-prefix-arg))
   (declare (special dtk-punctuation-mode dtk-speaker-process
-                    dtk-speak-driver-initialized
+                    dtk-speak-server-initialized
                     dtk-punctuation-mode-alist))
-  (when dtk-speak-driver-initialized
+  (when dtk-speak-server-initialized
     (cond
      (prefix
       (setq dtk-punctuation-mode mode )
       (setq-default dtk-punctuation-mode mode))
      (t (make-local-variable 'dtk-punctuation-mode)
         (setq dtk-punctuation-mode mode )))
-    (process-send-string dtk-speaker-process  
-                         (format "dectalk_set_punctuations %s\n"
-                                 mode))
+    (dtk-interp-set-punctuations mode)
     (message "set punctuation mode to %s %s"
              mode
              (if prefix "" "locally"))))
@@ -867,10 +930,9 @@ europe, spell', all of which can be turned on or off. "
  Typically used after the Dectalk has been power   cycled."
   (interactive)
   (declare (special  dtk-speaker-process
-                     dtk-speak-driver-initialized))
-  (when dtk-speak-driver-initialized
-    (process-send-string dtk-speaker-process
-                         (format "dectalk_reset \n"))))
+                     dtk-speak-server-initialized))
+  (when dtk-speak-server-initialized
+    (dtk-interp-reset-state)))
 
 ;;}}}
 ;;{{{  pause and resume 
@@ -884,21 +946,27 @@ perform a few actions before continuing to read a large document.  Emacspeak
 gives you speech feedback as usual once speech has been paused.  dtk-resume
 continues the interrupted speech irrespective of the buffer in which it is
 executed. Optional first  arg pauses speech even if there
-is no ongoing speech. This lets you flush paused speech without having to resume it."
+is no ongoing speech. This lets you flush paused speech
+without having to resume it.
+If speech has already been paused once, this command will
+not pause speech again-- instead you will hear an auditory
+warning if have auditory icons turned on."
   (interactive "P")
   (declare (special dtk-speaker-process
                     dtk-paused))
-  (when (or  prefix
-             (interactive-p)
-             (not dtk-paused))
-    (process-send-string dtk-speaker-process "dectalk_pause\n")
+  (cond
+   ((or  prefix (not dtk-paused))
+    (dtk-interp-pause)
     (setq dtk-paused t)
-    (emacspeak-auditory-icon 'button)))
+    (emacspeak-auditory-icon 'button)
+    (when  prefix
+      (and (interactive-p)
+	   (dtk-speak "Flushed previously paused speech "))))
+   ((interactive-p) (emacspeak-auditory-icon 'warn-user))))
 
 (defvar dtk-resume-should-toggle nil
   "*T means dtk-resume acts as a toggle.")
 
-
 (defun dtk-resume ()
   "Resumes  speech that has been suspended by executing
 command dtk-pause bound to \\[dtk-pause].
@@ -911,9 +979,9 @@ and variable dtk-resume-should-toggle is t
                     dtk-paused))
   (cond
    ((and dtk-resume-should-toggle
-         (not dtk-paused) )
+         (not dtk-paused))
     (dtk-pause))
-   (t (process-send-string dtk-speaker-process "dectalk_resume\n")
+   (t (dtk-interp-resume)
       (emacspeak-auditory-icon 'button)
       (setq dtk-paused nil))))
 
