@@ -1,4 +1,5 @@
-;;; $Id: emacspeak-advice.el,v 7.0 1997/11/13 15:32:30 raman Exp $
+;;; emacspeak-advice.el --- Advice all core Emacs functionality to speak intelligently
+;;; $Id: emacspeak-advice.el,v 8.0 1998/05/01 14:15:46 raman Exp $
 ;;; $Author: raman $ 
 ;;; Description:  Core advice forms that make emacspeak work
 ;;; Keywords: Emacspeak, Speech, Advice, Spoken  output
@@ -7,15 +8,15 @@
 ;;; LCD Archive Entry:
 ;;; emacspeak| T. V. Raman |raman@adobe.com 
 ;;; A speech interface to Emacs |
-;;; $Date: 1997/11/13 15:32:30 $ |
-;;;  $Revision: 7.0 $ | 
+;;; $Date: 1998/05/01 14:15:46 $ |
+;;;  $Revision: 8.0 $ | 
 ;;; Location undetermined
 ;;;
 
 ;;}}}
 ;;{{{  Copyright:
 
-;;;Copyright (C) 1995, 1996, 1997  T. V. Raman  Adobe Systems Incorporated
+;;;Copyright (C) 1995, 1996, 1997, 1998   T. V. Raman  Adobe Systems Incorporated
 ;;; Copyright (c) 1995, 1996,  1997 by T. V. Raman Adobe Systems Incorporated 
 ;;; All Rights Reserved. 
 ;;;
@@ -70,7 +71,7 @@
 (defadvice forward-word (after emacspeak pre act)
   "Speak the word you just moved to. "
   (when (interactive-p)
-      (skip-syntax-forward  " ")
+    (skip-syntax-forward  " ")
     (emacspeak-speak-word )))
 
 (defadvice backward-word (after emacspeak pre act)
@@ -263,6 +264,96 @@
   (when (interactive-p)
     (emacspeak-speak-line)))
 
+;;}}}
+;;{{{ Advise modify case commands to speak
+
+(defadvice upcase-word (around emacspeak pre act comp)
+  "Provide a tone to indicate that we upper cased the current
+word.  Speak the word that point lands on after the action
+is done.  If upcase-word is called with a negative argument,
+then point does not move. In this case, we speak the words
+that were upper cased."
+
+  (cond 
+   ((interactive-p)
+    (when dtk-stop-immediately (dtk-stop))
+    (let ((dtk-stop-immediately nil))
+      (dtk-tone 800 50 'force)
+      (cond
+       ((and (numberp current-prefix-arg)
+             (minusp current-prefix-arg))
+        ad-do-it
+        (let ((start (point)))
+          (save-excursion
+            (forward-word current-prefix-arg)
+            (emacspeak-speak-region start (point)))))
+       (t ad-do-it 
+          (save-excursion 
+            (skip-syntax-forward " ")
+            (if(eobp)
+                (message "Upper cased final word in buffer")
+              (emacspeak-speak-word)))))))
+   (t ad-do-it))
+  ad-return-value)
+
+(defadvice downcase-word (around emacspeak pre act comp)
+  "Provide a tone to indicate that we down cased the current
+word.  Speak the word that point lands on after the action
+is done.  If downcase-word is called with a negative
+argument, then point does not move. In this case, we speak
+the words that were down cased."
+
+  (cond 
+   ((interactive-p)
+    (when dtk-stop-immediately (dtk-stop))
+    (let ((dtk-stop-immediately nil))
+      (dtk-tone 600 50 'force)
+      (cond
+       ((and (numberp current-prefix-arg)
+             (minusp current-prefix-arg))
+        ad-do-it
+        (let ((start (point)))
+          (save-excursion
+            (forward-word current-prefix-arg)
+            (emacspeak-speak-region start (point)))))
+       (t ad-do-it 
+          (save-excursion 
+            (skip-syntax-forward " ")
+            (if(eobp)
+                (message "Lower cased final word in buffer")
+              (emacspeak-speak-word)))))))
+   (t ad-do-it))
+  ad-return-value)
+
+
+(defadvice capitalize-word (around emacspeak pre act comp)
+  "Provide a tone to indicate that we capitalized the current
+word.  Speak the word that point lands on after the action
+is done.  If capitalize-word is called with a negative
+argument, then point does not move. In this case, we speak
+the words that were capitalized."
+  (cond 
+   ((interactive-p)
+    (when dtk-stop-immediately (dtk-stop))
+    (let ((dtk-stop-immediately nil))
+      (dtk-tone 700 50 'force)
+      (cond
+       ((and (numberp current-prefix-arg)
+             (minusp current-prefix-arg))
+        ad-do-it
+        (let ((start (point)))
+          (save-excursion
+            (forward-word current-prefix-arg)
+            (emacspeak-speak-region start (point)))))
+       (t ad-do-it 
+          (save-excursion 
+            (skip-syntax-forward " ")
+            (if(eobp)
+                (message "Capitalized final word in buffer")
+              (emacspeak-speak-word)))))))
+   (t ad-do-it))
+  ad-return-value)
+
 ;;}}}
 ;;{{{  Advice deletion commands:
 
@@ -295,12 +386,12 @@
   "Speak character you're deleting"
   (cond
    ((interactive-p )
-      (dtk-tone 500 30 'force)
-      (and emacspeak-delete-char-speak-deleted-char
-           (emacspeak-speak-char t))
-      ad-do-it
-      (and emacspeak-delete-char-speak-current-char
-           (emacspeak-speak-char t)))
+    (dtk-tone 500 30 'force)
+    (and emacspeak-delete-char-speak-deleted-char
+         (emacspeak-speak-char t))
+    ad-do-it
+    (and emacspeak-delete-char-speak-current-char
+         (emacspeak-speak-char t)))
    (t ad-do-it))
   ad-return-value)
 
@@ -308,12 +399,12 @@
   "Speak character you're deleting."
   (cond
    ((interactive-p )
-      (dtk-tone 500 30 'force)
-      (and emacspeak-backward-delete-char-speak-deleted-char
-           (emacspeak-speak-this-char (preceding-char )))
-      ad-do-it
-      (and emacspeak-backward-delete-char-speak-current-char
-           (emacspeak-speak-this-char (preceding-char ))))
+    (dtk-tone 500 30 'force)
+    (and emacspeak-backward-delete-char-speak-deleted-char
+         (emacspeak-speak-this-char (preceding-char )))
+    ad-do-it
+    (and emacspeak-backward-delete-char-speak-current-char
+         (emacspeak-speak-this-char (preceding-char ))))
    (t ad-do-it))
   ad-return-value)
 
@@ -360,7 +451,7 @@
     (when dtk-stop-immediately (dtk-stop))
     (let ((dtk-stop-immediately nil))
       (dtk-tone 500 30)
-    (emacspeak-speak-line 1))))
+      (emacspeak-speak-line 1))))
 
 (defadvice kill-sexp (before emacspeak pre act )
   "Speak the sexp you killed. "
@@ -368,8 +459,8 @@
     (emacspeak-auditory-icon 'delete-object)
     (when dtk-stop-immediately (dtk-stop))
     (let ((dtk-stop-immediately nil))
-    (dtk-tone 500 30)
-    (emacspeak-speak-sexp 1 ))))
+      (dtk-tone 500 30)
+      (emacspeak-speak-sexp 1 ))))
 
 (defadvice kill-sentence (before emacspeak pre act )
   "Speak the line  you killed. "
@@ -377,8 +468,8 @@
     (emacspeak-auditory-icon 'delete-object)
     (when dtk-stop-immediately (dtk-stop))
     (let ((dtk-stop-immediately nil))
-    (dtk-tone 500 30)
-    (emacspeak-speak-line 1 ))))
+      (dtk-tone 500 30)
+      (emacspeak-speak-line 1 ))))
 
 (defadvice delete-blank-lines (before   emacspeak  pre act )
   "Provide auditory feedback"
@@ -415,10 +506,10 @@
            (emacspeak-self-insert-command last-input-char ))))
 
 (defadvice completion-separator-self-insert-autofilling (around fix-bug pre act comp)
-    (condition-case nil
-        ad-do-it
-      (error (set-syntax-table cmpl-saved-syntax)
-       (emacspeak-self-insert-command last-input-char ))))
+  (condition-case nil
+      ad-do-it
+    (error (set-syntax-table cmpl-saved-syntax)
+           (emacspeak-self-insert-command last-input-char ))))
 
 (defadvice completion-separator-self-insert-autofilling (after emacspeak pre act)
   "Speak what wascompleted followed by the next completion."
@@ -440,9 +531,9 @@
 
 (defadvice quoted-insert  (after emacspeak pre act )
   "Speak the character that was inserted."
-(when (interactive-p)
-(emacspeak-speak-this-char
-(preceding-char ))))
+  (when (interactive-p)
+    (emacspeak-speak-this-char
+     (preceding-char ))))
 
 ;;}}}
 ;;{{{  advice minibuffer to speak
@@ -480,20 +571,29 @@
 
 (defvar emacspeak-lazy-message-time 0
   "Records when we last spoke a message. ")
+(defvar emacspeak-speak-messages-should-pause-ongoing-speech
+  t
+"*If t then all messages will pause ongoing speech if any 
+before the message is spoken.")
 
 (defadvice message (around  emacspeak pre act)
   "Speak the message. "
-  (declare (special emacspeak-last-message
-                    emacspeak-speak-messages emacspeak-lazy-message-time))
+  (declare
+   (special emacspeak-last-message
+            emacspeak-speak-messages-should-pause-ongoing-speech
+            emacspeak-speak-messages emacspeak-lazy-message-time))
   (let ((dtk-stop-immediately t ))
     ad-do-it
     (setq emacspeak-last-message ad-return-value )
     (when (and   emacspeak-speak-messages ; speaking messages
-                 ad-return-value ;we really do have a message
+                 ad-return-value        ;we really do have a message
                  (/= emacspeak-lazy-message-time;; previous message not recent 
                      (setq emacspeak-lazy-message-time  (nth 1 (current-time)))))
-      ;; so we really need to speak it 
-                                 (dtk-speak ad-return-value))))
+      ;; so we really need to speak it
+      (when
+          emacspeak-speak-messages-should-pause-ongoing-speech
+        (dtk-pause))
+      (dtk-speak ad-return-value))))
 
 (defvar emacspeak-ange-ftp-last-percent nil
   "Cache the last percentage that emacspeak spoke.")
@@ -506,8 +606,8 @@
   (let ((emacspeak-speak-messages nil ))
     ad-do-it
     (when (or (null emacspeak-ange-ftp-last-percent)
-        (>= (abs (- ange-ftp-last-percent emacspeak-ange-ftp-last-percent ))
-           5))
+              (>= (abs (- ange-ftp-last-percent emacspeak-ange-ftp-last-percent ))
+                  5))
       (setq emacspeak-ange-ftp-last-percent ange-ftp-last-percent )
       (emacspeak-auditory-icon 'progress)
       (dtk-speak
@@ -517,13 +617,13 @@
 (defadvice signal (before emacspeak pre act compile)
   "Speak the error message as well."
   (dtk-pause)
-    (dtk-speak
-     (format "%s %s" 
-             (or (get (ad-get-arg 0) 'error-message)
-                 "Peculiar error ")
-             (mapconcat 'identity
-                        (ad-get-arg 1)
-                        " "))))
+  (dtk-speak
+   (format "%s %s" 
+           (or (get (ad-get-arg 0) 'error-message)
+               "Peculiar error ")
+           (mapconcat 'identity
+                      (ad-get-arg 1)
+                      " "))))
 
 (defadvice error (before emacspeak pre act)
   "Speak the error message.
@@ -531,16 +631,20 @@
   (let ((dtk-stop-immediately nil ))
     (dtk-pause)
     (emacspeak-auditory-icon 'warn-user)
-                    (dtk-speak (apply 'format  (ad-get-args 0)))))
+    (dtk-speak (apply 'format  (ad-get-args 0)))))
 
 (defadvice read-from-minibuffer (around emacspeak pre act)
   "Prompt using speech as well. "
   (let((prompt (ad-get-arg 0))
-       (default (ad-get-arg 1 )))
+       (initial (ad-get-arg 1 ))
+       (default (ad-get-arg 5)))
     (dtk-speak
-     (format "%s  %s"
+     (format "%s  %s%s"
              prompt
-             (if  default
+             (if  initial
+                 initial
+               "")
+             (if default
                  (format "Default: %s" default)
                "")))
     ad-do-it
@@ -579,6 +683,8 @@
   "Use speech when prompting.
   Produce an auditory icon if possible."
   (emacspeak-auditory-icon 'ask-short-question )
+  (when emacspeak-speak-messages-should-pause-ongoing-speech
+    (dtk-pause))
   (dtk-speak (format "%s  y or n" (ad-get-arg  0 )))
   ad-do-it
   (cond
@@ -593,6 +699,8 @@
   "Use speech when prompting.
   Produce an auditory icon as well. "
   (emacspeak-auditory-icon 'ask-question)
+  (when emacspeak-speak-messages-should-pause-ongoing-speech
+    (dtk-pause))
   (dtk-speak (format "%s  yes or no" (ad-get-arg  0 )))
   ad-do-it
   (cond
@@ -610,11 +718,15 @@
   "Prompt using speech. "
   (let ((dtk-stop-immediately t )
         (prompt (ad-get-arg 0))
-        (initial (ad-get-arg 4 )))
+        (initial (ad-get-arg 4 ))
+        (default (ad-get-arg 6)))
     (dtk-speak 
-     (format "%s %s"
+     (format "%s %s%s"
              (or prompt " ")
-             (or initial " ")))
+             (or initial " ")
+             (if default
+                 (format "Default: %s" default)
+               "")))
     ad-do-it
     (dtk-speak (format "%s" ad-return-value ))
     ad-return-value ))
@@ -732,14 +844,14 @@ in completion buffers"
     (emacspeak-kill-buffer-carefully "*Completions*")
     ad-do-it
     (let ((completions-buffer (get-buffer "*Completions*")))
-    (if (> (point) prior)
-      (dtk-speak (buffer-substring prior (point )))
-    (when (and completions-buffer
-           (window-live-p (get-buffer-window completions-buffer )))
-      (save-excursion
-        (set-buffer completions-buffer )
-        (emacspeak-prepare-completions-buffer)
-        (dtk-speak (buffer-string ))))))
+      (if (> (point) prior)
+          (dtk-speak (buffer-substring prior (point )))
+        (when (and completions-buffer
+                   (window-live-p (get-buffer-window completions-buffer )))
+          (save-excursion
+            (set-buffer completions-buffer )
+            (emacspeak-prepare-completions-buffer)
+            (dtk-speak (buffer-string ))))))
     ad-return-value))
 
 (defadvice minibuffer-complete (around emacspeak pre act)
@@ -800,9 +912,27 @@ in completion buffers"
   "Provide auditory feedback"
   (when (interactive-p)
     (emacspeak-auditory-icon 'close-object)
-    (emacspeak-speak-mode-line )))
+    (emacspeak-speak-line)))
 
-(defadvice minibuffer-complete-and-exit (before emacspeak pre act comp)
+(defadvice minibuffer-message (around emacspeak pre act comp)
+  "Speak the message if appropriate"
+  (declare (special emacspeak-last-message
+                    emacspeak-speak-messages emacspeak-lazy-message-time))
+  (let ((dtk-stop-immediately t ))
+    ad-do-it
+    (setq emacspeak-last-message ad-return-value )
+    (when (and   emacspeak-speak-messages ; speaking messages
+                 ad-return-value        ;we really do have a message
+                 (/= emacspeak-lazy-message-time;; previous message not recent 
+                     (setq emacspeak-lazy-message-time  (nth 1 (current-time)))))
+      ;; so we really need to speak it 
+      (dtk-speak ad-return-value))))
+
+
+;;}}}
+;;{{{ tmm support 
+
+(defadvice minibuffer-complete-and-exit (after emacspeak pre act comp)
   "Provide an auditory icon"
   (when (interactive-p)
     (emacspeak-auditory-icon 'button)))
@@ -812,12 +942,48 @@ in completion buffers"
   (when (interactive-p)
     (emacspeak-auditory-icon 'open-object )))
 
+(defadvice tmm-add-prompt (around emacspeak pre act comp)
+  "Speaks the list of completions we have available. "
+  ad-do-it
+  (let ((cap "cap "))
+    (put-text-property 0 (length cap)
+                       'personality 'paul-animated cap)
+    (put-text-property 0  (length tmm-mid-prompt)
+                       'personality 'inaudible
+                       tmm-mid-prompt)
+    (dtk-speak 
+     (mapconcat 
+      (function 
+       (lambda (choice)
+         (declare (special cap))
+         (let ((string (car choice )))
+           (put-text-property 
+            0 1 
+            'personality 'paul-animated string)
+           (when (string-match "^[A-Z]" string)
+             (setq string 
+                   (concat cap string)))
+           string)))
+      minibuffer-completion-table " ")
+     ))
+  ad-return-value)
+
+(defadvice tmm-shortcut (after emacspeak pre act comp)
+  "Provide contextual feedback when exitting minibuffer"
+  (emacspeak-auditory-icon 'button))
+
 ;;}}}
 ;;{{{  Advice comint:
 
 (add-hook 'comint-mode-hook
           (function (lambda ()
                       (modify-syntax-entry 10 ">"  ))))
+(defadvice comint-copy-old-input (after emacspeak pre act
+                                        comp)
+  "Provide auditory feedback"
+  (when (interactive-p)
+    (emacspeak-auditory-icon 'yank-object)
+    (emacspeak-speak-line)))
 
 (defadvice comint-output-filter (around emacspeak pre act)
   "Make comint speak its output. "
@@ -834,21 +1000,29 @@ in completion buffers"
                (dtk-stop ))))
     ad-return-value))
 
+(defadvice comint-dynamic-list-completions(around emacspeak pre act comp)
+  "Replacing mouse oriented completer with keyboard friendly equivalent"
+  (with-output-to-temp-buffer "*Completions*"
+    (display-completion-list (sort completions 'string-lessp)))
+  (switch-to-buffer (get-buffer "*Completions*"))
+  (next-completion 1)
+  (dtk-speak (emacspeak-get-current-completion-from-completions)))
+  
 (defadvice comint-dynamic-complete (around emacspeak pre act)
   "Say what you completed."
-  (let ((prior (point )))
+  (cond
+   ((interactive-p)
     (emacspeak-kill-buffer-carefully "*Completions*")
-    ad-do-it
-    (let ((completions-buffer (get-buffer "*Completions*"))
-          (dtk-stop-immediately t))
+    (let ((prior (point )))
+      ad-do-it
       (if (> (point) prior)
-          (dtk-speak (buffer-substring prior (point )))
+          (dtk-speak (buffer-substring prior (point ))))
+      (let ((completions-buffer (get-buffer "*Completions*")))
         (when (and completions-buffer
                    (window-live-p (get-buffer-window completions-buffer )))
-          (save-excursion
-            (set-buffer completions-buffer )
-            (dtk-speak (buffer-string ))))))
-    ad-return-value))
+          (emacspeak-auditory-icon 'help)))))
+   (t ad-do-it))
+  ad-return-value)
 
 (defadvice comint-next-input (after emacspeak pre act)
   "Speak the line. "
@@ -885,7 +1059,7 @@ in completion buffers"
     (emacspeak-auditory-icon 'large-movement)
     (if (eolp)
         (emacspeak-speak-line)
-    (emacspeak-speak-line 1))))
+      (emacspeak-speak-line 1))))
 
 (defadvice comint-next-prompt (after emacspeak pre act )
   "Provide spoken feedback"
@@ -893,13 +1067,35 @@ in completion buffers"
     (emacspeak-auditory-icon 'large-movement)
     (if (eolp)
         (emacspeak-speak-line)
-    (emacspeak-speak-line 1))))
+      (emacspeak-speak-line 1))))
 
-(defadvice comint-dynamic-list-input-ring (after emacspeak pre act)
-  "Provide auditory feedback"
-  (when (interactive-p)
-    (emacspeak-auditory-icon 'help)
-  (message  "Switch to the other window to browse the input history ")))
+(defadvice comint-dynamic-list-input-ring (around emacspeak pre act comp)
+  "List in help buffer the buffer's input history."
+  (cond
+   ((interactive-p)
+    (if (or (not (ring-p comint-input-ring))
+            (ring-empty-p comint-input-ring))
+        (message "No history")
+      (let ((history nil)
+            (history-buffer " *Input History*")
+            (index (1- (ring-length comint-input-ring))))
+        ;; We have to build up a list ourselves from the ring vector.
+        (while (>= index 0)
+          (setq history (cons (ring-ref comint-input-ring index) history)
+                index (1- index)))
+        ;; Change "completion" to "history reference"
+        ;; to make the display accurate.
+        (with-output-to-temp-buffer history-buffer
+          (display-completion-list history)
+          (switch-to-buffer history-buffer)
+          (forward-line 3)
+          (while (search-backward "completion" nil 'move)
+            (replace-match "history reference")))
+        (emacspeak-auditory-icon 'help)
+        (next-completion 1)
+        (dtk-speak (emacspeak-get-current-completion-from-completions)))))
+   (t ad-do-it))
+  ad-return-value)
 
 (defadvice comint-kill-output (after emacspeak pre act )
   "Provide auditory feedback"
@@ -926,16 +1122,14 @@ in completion buffers"
   "Provide spoken feedback"
   (when (interactive-p)
     (let ((pmark (process-mark (get-buffer-process (current-buffer)))))
-    (when  (> (point) (marker-position pmark))
-      (emacspeak-auditory-icon 'delete-object )
+      (when  (> (point) (marker-position pmark))
+        (emacspeak-auditory-icon 'delete-object )
 	(emacspeak-speak-region  pmark (point))))))
 
 (defadvice comint-dynamic-list-filename-completions (after emacspeak pre act )
   "Provide auditory feedback"
   (when (interactive-p)
-    (emacspeak-auditory-icon 'help)
-    (message "Switch to the completions window to browse the possible
-completions for filename at point")))
+    (emacspeak-auditory-icon 'help)))
 
 ;;}}}
 ;;{{{  Advice centering and filling commands:
@@ -972,6 +1166,11 @@ completions for filename at point")))
   (when (interactive-p)
     (emacspeak-auditory-icon 'fill-object )
     (message "Filled current paragraph")))
+(defadvice indent-relative (after emacspeak pre act comp)
+  "Provide auditory feedback to indicate indentation"
+  (when (interactive-p)
+    (emacspeak-auditory-icon 'large-movement)
+    (emacspeak-speak-current-column)))
 
 (defadvice indent-sexp  (after emacspeak pre act)
   "Provide auditory feedback"
@@ -999,7 +1198,7 @@ completions for filename at point")))
     (cond
      ((and vc-mode
            (stringp vc-mode))
-           (substring id 5  nil ))
+      (substring id 5  nil ))
      (t " "))))
 
 ;;; Advice for most used vc functions:
@@ -1009,13 +1208,13 @@ completions for filename at point")))
   (cond
    ((interactive-p)
     (let ((message (format  "Checking %s version %s "
-           (if buffer-read-only  "out previous " " in new  ")
-           (emacspeak-vc-get-version-id))))
-  (if buffer-read-only 
-      (emacspeak-auditory-icon 'open-object )
-    (emacspeak-auditory-icon 'close-object))
-  ad-do-it
-  (message message )))
+                            (if buffer-read-only  "out previous " " in new  ")
+                            (emacspeak-vc-get-version-id))))
+      (if buffer-read-only 
+          (emacspeak-auditory-icon 'open-object )
+        (emacspeak-auditory-icon 'close-object))
+      ad-do-it
+      (message message )))
    (t ad-do-it ))
   ad-return-value )
 
@@ -1044,7 +1243,7 @@ completions for filename at point")))
   (when (interactive-p)
     (emacspeak-auditory-icon  'close-object)
     (message "Checked   in  version %s "
-           (emacspeak-vc-get-version-id))))
+             (emacspeak-vc-get-version-id))))
 
 (progn
   (require 'vc)
@@ -1073,11 +1272,11 @@ completions for filename at point")))
     (emacspeak-speak-mode-line)))
 
 (defadvice not-modified (after emacspeak pre act )
-"Provide an auditory icon"
-(when (interactive-p)
-(if (ad-get-arg 0)
-(emacspeak-auditory-icon 'modified-object )
-(emacspeak-auditory-icon 'unmodified-object))))
+  "Provide an auditory icon"
+  (when (interactive-p)
+    (if (ad-get-arg 0)
+        (emacspeak-auditory-icon 'modified-object )
+      (emacspeak-auditory-icon 'unmodified-object))))
 
 (defadvice comment-region (after emacspeak pre act )
   "Provide spoken feedback."
@@ -1110,6 +1309,17 @@ completions for filename at point")))
    (t ad-do-it))
   ad-return-value)
 
+(defadvice save-some-buffers (around emacspeak pre act)
+  "Produce an auditory icon if possible. "
+  (declare (special emacspeak-last-message))
+  (cond
+   ((interactive-p)
+    (setq emacspeak-last-message nil)
+    ad-do-it
+    (emacspeak-auditory-icon 'save-object))
+   (t ad-do-it))
+  ad-return-value)
+
 (defadvice kill-region (around emacspeak pre act)
   "Indicate region has been killed. 
   Use an auditory icon if possible. "
@@ -1118,8 +1328,8 @@ completions for filename at point")))
     (let ((count (count-lines (region-beginning)
                               (region-end))))
       ad-do-it
-    (emacspeak-auditory-icon 'delete-object )
-    (message "Killed region containing %s lines" count)))
+      (emacspeak-auditory-icon 'delete-object )
+      (message "Killed region containing %s lines" count)))
    (t ad-do-it))
   ad-return-value)
     
@@ -1156,8 +1366,8 @@ completions for filename at point")))
 (defadvice kill-buffer (before emacspeak pre act)
   "Speak the prompt, please:"
   (when (interactive-p )
-      (dtk-speak (format "Kill buffer:  default  %s" 
-                         (buffer-name)))))
+    (dtk-speak (format "Kill buffer:  default  %s" 
+                       (buffer-name)))))
 
 (defadvice kill-buffer (after emacspeak pre act)
   " Produce an auditory icon to indicate closing of an object.
@@ -1174,6 +1384,28 @@ completions for filename at point")))
   (when (interactive-p)
     (emacspeak-auditory-icon 'select-object)
     (emacspeak-speak-mode-line)))
+(defadvice delete-frame (after emacspeak pre act comp)
+  "Provide auditory feedback about frame and buffer that
+become current"
+  (when (interactive-p)
+    (emacspeak-speak-mode-line)
+    (emacspeak-auditory-icon 'close-object)))
+
+(defadvice make-frame-command (after emacspeak pre act comp)
+  "Indicate that a new frame is being created"
+  (when (interactive-p)
+    (emacspeak-auditory-icon 'open-object)
+    (emacspeak-speak-mode-line)))
+
+
+(defadvice other-frame (after emacspeak pre act )
+  "Speak modeline.
+  Indicate change of selection with an auditory icon 
+  if possible. "
+  (when (interactive-p)
+    (emacspeak-auditory-icon 'select-object)
+    (emacspeak-speak-mode-line)))
+
 
 (defadvice move-to-window-line (after emacspeak pre act comp)
   "Provide auditory feedback"
@@ -1183,7 +1415,7 @@ completions for filename at point")))
 
 (defadvice execute-extended-command (before emacspeak pre act)
   "Prompt using speech. "
-  (dtk-speak "Execute command "))
+  (dtk-say "Command "))
 
 (defadvice rename-buffer  (around emacspeak pre act)
   "Provide spoken feedback"
@@ -1249,6 +1481,15 @@ completions for filename at point")))
   "Speak the help"
   (when (interactive-p) (emacspeak-speak-help )))
 
+(defadvice describe-key (after emacspeak pre act)
+  "Speak the help"
+  (when (interactive-p)
+    (emacspeak-speak-help )))
+(add-hook 'help-mode-hook
+          (function
+           (lambda nil
+             (modify-syntax-entry 10 " "))))
+
 (defadvice help-with-tutorial (after emacspeak pre act comp)
   "Provide auditory feedback"
   (when (interactive-p)
@@ -1256,22 +1497,40 @@ completions for filename at point")))
     (message "Welcome to the Emacs tutorial")))
 
 (defadvice key-description (around emacspeak pre act )
+  (declaim (special case-fold-search))
   (let ((emacspeak-scratch (get-buffer-create  " *dtk-scratch-buffer* "))
-        (ctrl-regexp "C-\\(.\\)"))
+        (shift-regexp "S-\\(.\\)")
+        (ctrl-regexp "C-\\(.\\)")
+        (meta-regexp "M-\\(.\\)")
+        (caps-regexp "\\b[A-Z]\\b"))
     ad-do-it
     (save-excursion
       (set-buffer emacspeak-scratch )
+      (setq case-fold-search nil)
       (erase-buffer)
-      (insert  (format "%s" ad-return-value ))
+      (insert  (format " %s " ad-return-value ))
       (goto-char (point-min))
       (save-match-data
         (while (search-forward "SPC"  nil t )
-          (replace-match "Space"))
+          (replace-match "space"))
+        (goto-char (point-min))
         (while (search-forward "ESC"  nil t )
           (replace-match "escape"))
         (goto-char (point-min))
+        (while (search-forward "RET"  nil t )
+          (replace-match "return"))
+        (goto-char (point-min))
+        (while (re-search-forward shift-regexp  nil t )
+          (replace-match "shift \\1"))        
+        (goto-char (point-min))
         (while (re-search-forward ctrl-regexp  nil t )
-          (replace-match "Control-\\1 ")))
+          (replace-match "control \\1"))
+        (goto-char (point-min))
+        (while (re-search-forward meta-regexp  nil t )
+          (replace-match "meta \\1"))
+        (goto-char (point-min))
+        (while (re-search-forward caps-regexp nil t)
+          (replace-match " cap \\& " t)))
       (setq ad-return-value (buffer-string )))
     ad-return-value))
 
@@ -1282,46 +1541,68 @@ Auditory highlight indicates position of point."
   (when (interactive-p)
     (emacspeak-auditory-icon 'large-movement )
     (ems-set-personality-temporarily  (point) (1+ (point))
-                                                  'paul-animated
-    (emacspeak-speak-line))))
+                                      'paul-animated
+                                      (emacspeak-speak-line))))
 
 (defadvice newline (before emacspeak pre act)
-  "Speak the previous line if line echo is on. 
-  See command \\[emacspeak-toggle-line-echo]. "
-  (declare (special emacspeak-line-echo ))
-  (when (and emacspeak-line-echo 
-	     (interactive-p))
-    (emacspeak-speak-line )))
+  "Speak the previous line if line echo is on.  See command
+\\[emacspeak-toggle-line-echo].  Otherwise cue the user to
+the newly created blank line. "
 
-(defadvice newline-and-indent (before emacspeak pre act)
-  "Speak the previous line if line echo is on. 
-
-  See command \\[emacspeak-toggle-line-echo]. "
   (declare (special emacspeak-line-echo ))
-  (when (and emacspeak-line-echo 
-	     (interactive-p))
-    (emacspeak-speak-line )))
+  (when (interactive-p)
+    (cond 
+     (emacspeak-line-echo 
+      (emacspeak-speak-line ))
+     (t(if dtk-stop-immediately (dtk-stop))
+       (dtk-tone 200   250 'force)))))
+
+(defadvice newline-and-indent (around emacspeak pre act)
+  "Speak the previous line if line echo is on. 
+  See command \\[emacspeak-toggle-line-echo].
+Otherwise cue user to the line just created. "
+  (declare (special emacspeak-line-echo ))
+  (cond
+   ((interactive-p)
+    (cond
+     (emacspeak-line-echo 
+      (emacspeak-speak-line )
+      ad-do-it)
+     (t ad-do-it       
+        (dtk-speak-using-voice 'annotation-voice
+                               (format
+                                "indent %s"
+                                (current-column)))
+        (dtk-force))))
+   (t ad-do-it))
+  ad-return-value)
 
 (defadvice keyboard-quit (before emacspeak pre act)
   "Stop speech first. "
   (dtk-pause)
   (emacspeak-auditory-icon 'warn-user))
 
+(defadvice keyboard-escape-quit (before emacspeak pre act)
+  "Stop speech first. "
+  (dtk-pause)
+  (emacspeak-auditory-icon 'item)
+  (emacspeak-speak-mode-line))
+
 (defadvice delete-indentation (after emacspeak pre act)
   "Speak the line. "
   (when (interactive-p) (emacspeak-speak-line)))
 
 (defadvice eval-last-sexp (after emacspeak pre act)
   "Also speaks the result of evaluation."
- (let ((dtk-chunk-separator-syntax " .<>()$\"\'"))
-  (dtk-speak 
-   (format "%s" ad-return-value ))))
+  (let ((dtk-chunk-separator-syntax " .<>()$\"\'"))
+    (dtk-speak 
+     (format "%s" ad-return-value ))))
 
 (defadvice eval-expression (after emacspeak pre act)
   "Also speaks the result of evaluation."
   (let ((dtk-chunk-separator-syntax " .<>()$\"\'"))
-  (dtk-speak 
-   (format "%s" ad-return-value ))))
+    (dtk-speak 
+     (format "%s" ad-return-value ))))
 
 (defadvice shell (after emacspeak pre act )
   "announce switching to shell mode.
@@ -1358,10 +1639,10 @@ Auditory highlight indicates position of point."
 (defadvice mail-other-frame (after emacspeak pre act)
   "Give some auditory feedback. "
   (emacspeak-auditory-icon 'open-object)
-(let ((emacspeak-speak-messages nil))
-  (save-excursion
-    (beginning-of-buffer)
-    (emacspeak-speak-line))))
+  (let ((emacspeak-speak-messages nil))
+    (save-excursion
+      (beginning-of-buffer)
+      (emacspeak-speak-line))))
 
 (defadvice mail-subject (after emacspeak pre act)
   "Speak the subject line."
@@ -1559,10 +1840,10 @@ Auditory highlight indicates position of point."
   "Provide auditory feedback"
   (when (interactive-p)
     (ems-set-personality-temporarily
- (max (point-min) (1- (point)))
-                                     (min (point-max) (1+ (point)))
-                                     'harry 
-                                     (emacspeak-speak-line ))
+     (max (point-min) (1- (point)))
+     (min (point-max) (1+ (point)))
+     'harry 
+     (emacspeak-speak-line ))
     (if (buffer-modified-p)
         (emacspeak-auditory-icon 'modified-object)
       (emacspeak-auditory-icon 'unmodified-object ))))
@@ -1606,10 +1887,10 @@ Auditory highlight indicates position of point."
   (cond
    ((interactive-p)
     (let ((emacspeak-speak-messages nil))
-    (dtk-speak "Byte compiling ")
-    ad-do-it 
-    (emacspeak-auditory-icon 'task-done)
-    (dtk-speak "Done byte compiling ")))
+      (dtk-speak "Byte compiling ")
+      ad-do-it 
+      (emacspeak-auditory-icon 'task-done)
+      (dtk-speak "Done byte compiling ")))
    (t ad-do-it))
   ad-return-value)
 
@@ -1665,7 +1946,7 @@ Auditory highlight indicates position of point."
 
 (defadvice search-backward (before emacspeak pre act)
   "Prompt using speech"
-(cond
+  (cond
    ((interactive-p )
     (dtk-speak "Search backward  for ")
     ad-do-it
@@ -1684,42 +1965,46 @@ Auditory highlight indicates position of point."
 ;;{{{  temporarily disable message advice during searches. 
 
 (add-hook 'isearch-mode-hook 
-          (function (lambda () 
-                      (setq emacspeak-speak-messages nil))))
+          (function
+           (lambda () 
+             (setq emacspeak-speak-messages nil))))
+
 (add-hook 'isearch-mode-end-hook
-          (function (lambda ()
-                      (setq emacspeak-speak-messages t ))))
+          (function
+           (lambda ()
+             (setq emacspeak-speak-messages t ))))
 
 ;;}}}
 ;;{{{  Advice isearch-search to speak
+
 (defadvice isearch-forward (before emacspeak pre act comp)
-"Provide auditory feedback.
+  "Provide auditory feedback.
 Pause ongoing speech first."
-(when (interactive-p)
-(emacspeak-auditory-icon 'open-object)
-(dtk-pause)))
+  (when (interactive-p)
+    (emacspeak-auditory-icon 'open-object)
+    (dtk-pause)))
 
 (defadvice isearch-backward (before emacspeak pre act comp)
-"Provide auditory feedback.
+  "Provide auditory feedback.
 Pause ongoing speech first."
-(when (interactive-p)
-(emacspeak-auditory-icon 'open-object)
-(dtk-pause)))
+  (when (interactive-p)
+    (emacspeak-auditory-icon 'open-object)
+    (dtk-pause)))
 
 
 (defadvice isearch-forward-regexp (before emacspeak pre act comp)
-"Provide auditory feedback.
+  "Provide auditory feedback.
 Pause ongoing speech first."
-(when (interactive-p)
-(emacspeak-auditory-icon 'open-object)
-(dtk-pause)))
+  (when (interactive-p)
+    (emacspeak-auditory-icon 'open-object)
+    (dtk-pause)))
 
 (defadvice isearch-backward-regexp (before emacspeak pre act comp)
-"Provide auditory feedback.
+  "Provide auditory feedback.
 Pause ongoing speech first."
-(when (interactive-p)
-(emacspeak-auditory-icon 'open-object)
-(dtk-pause)))
+  (when (interactive-p)
+    (emacspeak-auditory-icon 'open-object)
+    (dtk-pause)))
 
 
 (defadvice isearch-search (after emacspeak pre act)
@@ -1736,6 +2021,22 @@ Pause ongoing speech first."
      'harry
      (emacspeak-speak-line nil ))))
 
+(defadvice isearch-done (around emacspeak pre act comp)
+  "Done searching --provide appropriate feedback"
+  (let ((emacspeak-speak-messages-should-pause-ongoing-speech nil))
+    ad-do-it
+    ))
+
+(defadvice isearch-exit (around emacspeak pre act comp)
+  "Done searching --provide appropriate feedback"
+  (let ((emacspeak-speak-messages-should-pause-ongoing-speech nil))
+    ad-do-it))
+
+(defadvice isearch-abort (around emacspeak pre act comp)
+  "Done searching --provide appropriate feedback"
+  (let ((emacspeak-speak-messages-should-pause-ongoing-speech nil))
+    ad-do-it))
+
 ;;}}}
 
 (defadvice isearch-yank-word (after emacspeak pre act comp)
@@ -1796,7 +2097,7 @@ message")
   "Never show the mark set message"
   (or (ad-get-arg 1)
       (memq last-command emacspeak-advice-smart-mark-functions)
-  (ad-set-arg 1 t))
+      (ad-set-arg 1 t))
   ad-do-it
   ad-return-value)
 
@@ -1807,7 +2108,7 @@ message")
     (emacspeak-auditory-icon 'mark-object )
     (ems-set-personality-temporarily (point) (1+ (point))
                                      'paul-animated
-                                       (emacspeak-speak-line ))))
+                                     (emacspeak-speak-line ))))
 
 (defadvice pop-global-mark (after emacspeak pre act)
   "Produce an auditory icon if possible. "
@@ -1815,7 +2116,7 @@ message")
     (emacspeak-auditory-icon 'mark-object )
     (ems-set-personality-temporarily (point) (1+ (point))
                                      'paul-animated
-                                       (emacspeak-speak-line ))
+                                     (emacspeak-speak-line ))
     (emacspeak-speak-mode-line)))
 
 (defadvice mark-defun (after emacspeak pre act)
@@ -1884,6 +2185,13 @@ message")
 
 ;;}}}
 ;;{{{  emacs registers
+(defadvice point-to-register (after emacspeak pre act comp)
+  "Produce auditory icon to indicate mark set"
+  (when (interactive-p)
+    (emacspeak-auditory-icon 'mark-object)
+    (if current-prefix-arg
+        (message "Stored current frame configuration")
+      (emacspeak-speak-line))))
 
 (defadvice copy-to-register (before emacspeak pre act)
   "Acknowledge the copy"
@@ -1916,6 +2224,12 @@ message")
   (when (interactive-p)
     (message "Copied window configuration to register %c"
              (ad-get-arg 0 ))))
+(defadvice frame-configuration-to-register (after emacspeak pre act )
+  "Provide auditory feedback"
+  (when (interactive-p)
+    (message "Copied window configuration to register %c"
+             (ad-get-arg 0 ))))
+
 
 ;;}}}
 ;;{{{  Modify syntax entries for modes where necessary:
@@ -1930,18 +2244,18 @@ message")
 (defun emacspeak-minibuffer-setup-hook ()
   "Actions to take when entering the minibuffer with emacspeak running."
   (declare (special dtk-stop-immediately))
-(emacspeak-auditory-icon 'open-object)
-(setq dtk-stop-immediately t))
-
+  (emacspeak-auditory-icon 'open-object)
+  (setq dtk-stop-immediately t))
+    
 (declaim (special minibuffer-setup-hook))
 (setq minibuffer-setup-hook 'emacspeak-minibuffer-setup-hook)
 
 (defun emacspeak-minibuffer-exit-hook ()
   "Actions performed when exiting the minibuffer with Emacspeak loaded."
-(emacspeak-auditory-icon 'close-object))
+  (emacspeak-auditory-icon 'close-object))
 
-;(declaim (special minibuffer-exit-hook))
-;(setq minibuffer-exit-hook 'emacspeak-minibuffer-exit-hook)
+                                        ;(declaim (special minibuffer-exit-hook))
+                                        ;(setq minibuffer-exit-hook 'emacspeak-minibuffer-exit-hook)
 
 ;;}}}
 ;;{{{ Advice occur
@@ -1956,14 +2270,14 @@ message")
 ;;{{{  abbrev mode advice
 
 (defadvice expand-abbrev (around emacspeak pre act comp)
-"Speak what you expanded"
-(let ((start (save-excursion
-(backward-word 1)
-(point))))
-ad-do-it
-(dtk-speak 
-(buffer-substring 
-start (point)))))
+  "Speak what you expanded"
+  (let ((start (save-excursion
+                 (backward-word 1)
+                 (point))))
+    ad-do-it
+    (dtk-speak 
+     (buffer-substring 
+      start (point)))))
 
 
 (defadvice abbrev-mode (after emacspeak pre act comp)
@@ -2002,10 +2316,11 @@ Do not change this unless you know what you are doing.")
 (declaim (special emacs-version))
 (when (string-lessp "19.34" emacs-version)
   ;;; first advice princ 
-(defadvice princ (after emacspeak pre  activate comp)
-  "Speak return value from princ if appropriate"
-  (when  emacspeak-advice-advice-princ
-  (dtk-speak ad-return-value)))
+
+  (defadvice princ (after emacspeak pre  activate comp)
+    "Speak return value from princ if appropriate"
+    (when  emacspeak-advice-advice-princ
+      (dtk-speak ad-return-value)))
 
   (defadvice describe-key-briefly (around emacspeak pre act comp)
     "Speak what you displayed"
@@ -2023,7 +2338,15 @@ Do not change this unless you know what you are doing.")
         ad-do-it))
      (t ad-do-it))
     ad-return-value)
-);;; end emacs 20 conditional 
+  );;; end emacs 20 conditional 
+
+;;}}}
+;;{{{ apropos and friends
+(defadvice apropos-follow (after emacspeak pre act comp)
+  "Speak the help you displayed"
+  (when (interactive-p)
+    (emacspeak-auditory-icon 'select-object)
+    (emacspeak-speak-help)))
 
 ;;}}}
 (provide 'emacspeak-advice)
