@@ -1,4 +1,5 @@
-;;; $Id: emacspeak-calendar.el,v 7.0 1997/11/13 15:32:30 raman Exp $
+;;; emacspeak-calendar.el --- Speech enable Emacs Calendar -- maintain a diary and appointments
+;;; $Id: emacspeak-calendar.el,v 8.0 1998/05/01 14:15:46 raman Exp $
 ;;; $Author: raman $ 
 ;;; Description:  Emacspeak extensions to speech enable the calendar.
 ;;; Keywords: Emacspeak, Calendar, Spoken Output
@@ -7,14 +8,14 @@
 ;;; LCD Archive Entry:
 ;;; emacspeak| T. V. Raman |raman@adobe.com 
 ;;; A speech interface to Emacs |
-;;; $Date: 1997/11/13 15:32:30 $ |
-;;;  $Revision: 7.0 $ | 
+;;; $Date: 1998/05/01 14:15:46 $ |
+;;;  $Revision: 8.0 $ | 
 ;;; Location undetermined
 ;;;
 
 ;;}}}
 ;;{{{  Copyright:
-;;;Copyright (C) 1995, 1996, 1997  T. V. Raman  Adobe Systems Incorporated
+;;;Copyright (C) 1995, 1996, 1997, 1998   T. V. Raman  Adobe Systems Incorporated
 ;;; Copyright (c) 1994, 1995 by Digital Equipment Corporation.
 ;;; All Rights Reserved. 
 ;;;
@@ -301,18 +302,11 @@
 
 (defun emacspeak-appt-speak-appointment (minutes-left new-time message )
   "Speak the appointment instead of displaying it visually."
-  (let ((appt-buffer (get-buffer-create " *appointments*")))
+  (let ((emacspeak-speak-messages-should-pause-ongoing-speech nil))
     (emacspeak-auditory-icon 'alarm)
-    (save-excursion
-      (set-buffer appt-buffer)
-      (set  (make-local-variable 'dtk-punctuation-mode ) "some")
-    (emacspeak-dtk-sync)
-    (erase-buffer)
-    (insert
-     (format "You have an appointment in %s minutes. %s"
-             minutes-left message ))
-    (message  (buffer-string )))
-    (emacspeak-dtk-sync)))
+    (dtk-pause t)
+    (message "You have an appointment in %s minutes. %s" minutes-left message )))
+      
 
 (declaim (special appt-disp-window-function))
 (setq appt-disp-window-function 'emacspeak-appt-speak-appointment)
