@@ -1,4 +1,6 @@
-;;; $Id: emacspeak-dired.el,v 7.0 1997/11/13 15:32:30 raman Exp $
+;;; emacspeak-dired.el --- Speech enable Dired Mode -- A powerful File Manager
+
+;;; $Id: emacspeak-dired.el,v 8.0 1998/05/01 14:15:46 raman Exp $
 ;;; $Author: raman $ 
 ;;; Description:  Emacspeak extension to speech enable dired 
 ;;; Keywords: Emacspeak, Dired, Spoken Output
@@ -7,14 +9,14 @@
 ;;; LCD Archive Entry:
 ;;; emacspeak| T. V. Raman |raman@adobe.com 
 ;;; A speech interface to Emacs |
-;;; $Date: 1997/11/13 15:32:30 $ |
-;;;  $Revision: 7.0 $ | 
+;;; $Date: 1998/05/01 14:15:46 $ |
+;;;  $Revision: 8.0 $ | 
 ;;; Location undetermined
 ;;;
 
 ;;}}}
 ;;{{{  Copyright:
-;;;Copyright (C) 1995, 1996, 1997  T. V. Raman  Adobe Systems Incorporated
+;;;Copyright (C) 1995, 1996, 1997, 1998   T. V. Raman  Adobe Systems Incorporated
 ;;; Copyright (c) 1994, 1995 by Digital Equipment Corporation.
 ;;; All Rights Reserved. 
 ;;;
@@ -78,6 +80,11 @@
 
 ;;}}}
 ;;{{{  advice:
+(defadvice dired-quit (after emacspeak pre act comp)
+  "Provide auditory feedback"
+  (when (interactive-p)
+    (emacspeak-auditory-icon 'close-object)
+    (emacspeak-speak-mode-line)))
 
 (defadvice dired (after emacspeak pre act)
 "Produce an auditory icon. "
@@ -234,17 +241,100 @@ unless dired-listing-switches contains -al"
                 (forward-line 1 ))))
         (setq buffer-read-only read-only )))))
 
+;;}}}
+;;{{{ Additional status speaking commands
+
+(defun emacspeak-dired-speak-header-line()
+  "Speak the header line of the dired buffer. "
+  (interactive)
+  (emacspeak-auditory-icon 'select-object)
+  (save-excursion (goto-char (point-min))
+    (forward-line 2)
+(emacspeak-speak-region (point-min) (point))))
+
+(defun emacspeak-dired-speak-file-size ()
+  "Speak the size of the current file"
+  (interactive)
+  (let ((filename (dired-get-filename nil t)))
+    (cond
+     (filename 
+      (emacspeak-auditory-icon 'select-object)
+      (message "File size %s"
+               (nth 7 (file-attributes filename ))))
+     (t (message "No file on current line")))))
+
+(defun emacspeak-dired-speak-file-modification-time ()
+  "Speak modification time  of the current file"
+  (interactive)
+  (let ((filename (dired-get-filename nil t)))
+    (cond
+     (filename 
+      (emacspeak-auditory-icon 'select-object)
+      (message "Modified on : %s"
+               (format-time-string 
+                emacspeak-speak-time-format-string
+                (nth 5 (file-attributes filename )))))
+     (t (message "No file on current line")))))
+
+(defun emacspeak-dired-speak-file-access-time ()
+  "Speak access time  of the current file"
+  (interactive)
+  (let ((filename (dired-get-filename nil t)))
+    (cond
+     (filename 
+      (emacspeak-auditory-icon 'select-object)
+      (message "Last accessed   on  %s"
+               (format-time-string 
+                emacspeak-speak-time-format-string
+                (nth 4 (file-attributes filename )))))
+     (t (message "No file on current line")))))
+(defun emacspeak-dired-speak-symlink-target ()
+  "Speaks the target of the symlink on the current line"
+  (interactive)
+  (let ((filename (dired-get-filename nil t)))
+    (cond
+     (filename 
+      (emacspeak-auditory-icon 'select-object)
+      (cond
+       ((file-symlink-p filename)
+        (message "Target is %s"
+                 (file-chase-links filename)))
+       (t (message "%s is not a symbolic link" filename))))
+     (t (message "No file on current line")))))
+(defun emacspeak-dired-speak-file-permissions ()
+  "Speak the permissions of the current file"
+  (interactive)
+  (let ((filename (dired-get-filename nil t)))
+    (cond
+     (filename 
+      (emacspeak-auditory-icon 'select-object)
+      (message "Permissions %s"
+               (nth 8 (file-attributes filename ))))
+     (t (message "No file on current line")))))
+
 ;;}}}
 ;;{{{  keys
 
 (add-hook 'dired-mode-hook
-          (function (lambda ()
-                      (declare (special dired-mode-map ))
-                      (define-key dired-mode-map "\C-i"
-                        'emacspeak-speak-next-field)
-                      (define-key dired-mode-map  "," 'emacspeak-speak-previous-field)
-                      (define-key dired-mode-map '[up] 'dired-previous-line)
-                      (define-key dired-mode-map '[down] 'dired-next-line))))
+          (function 
+           (lambda ()
+             (declare (special dired-mode-map ))
+             (define-key  dired-mode-map "/"
+               'emacspeak-dired-speak-file-permissions)                      
+             (define-key  dired-mode-map ";" 'emacspeak-dired-speak-header-line)
+             (define-key  dired-mode-map "a" 
+               'emacspeak-dired-speak-file-access-time)
+             (define-key dired-mode-map "c"
+               'emacspeak-dired-speak-file-modification-time)
+             (define-key dired-mode-map "z"
+               'emacspeak-dired-speak-file-size)
+             (define-key dired-mode-map "t"
+               'emacspeak-dired-speak-symlink-target)
+             (define-key dired-mode-map "\C-i"
+               'emacspeak-speak-next-field)
+             (define-key dired-mode-map  "," 'emacspeak-speak-previous-field)
+             (define-key dired-mode-map '[up] 'dired-previous-line)
+             (define-key dired-mode-map '[down] 'dired-next-line))))
 
 ;;}}}
 ;;{{{  fix interactive commands
