@@ -1,4 +1,5 @@
-;;; $Id: emacspeak-eterm.el,v 7.0 1997/11/13 15:32:30 raman Exp $
+;;; emacspeak-eterm.el --- Speech enable eterm -- Emacs' terminal emulator  term.el
+;;; $Id: emacspeak-eterm.el,v 8.0 1998/05/01 14:15:46 raman Exp $
 ;;; $Author: raman $ 
 ;;; Description:  Emacspeak extension to speech enable eterm. 
 ;;; Keywords: Emacspeak, Eterm, Terminal emulation, Spoken Output
@@ -7,14 +8,14 @@
 ;;; LCD Archive Entry:
 ;;; emacspeak| T. V. Raman |raman@adobe.com 
 ;;; A speech interface to Emacs |
-;;; $Date: 1997/11/13 15:32:30 $ |
-;;;  $Revision: 7.0 $ | 
+;;; $Date: 1998/05/01 14:15:46 $ |
+;;;  $Revision: 8.0 $ | 
 ;;; Location undetermined
 ;;;
 
 ;;}}}
 ;;{{{  Copyright:
-;;;Copyright (C) 1995, 1996, 1997  T. V. Raman  Adobe Systems Incorporated
+;;;Copyright (C) 1995, 1996, 1997, 1998   T. V. Raman  Adobe Systems Incorporated
 ;;; Copyright (c) 1994, 1995 by Digital Equipment Corporation.
 ;;; All Rights Reserved. 
 ;;;
@@ -118,7 +119,9 @@
    ;(define-key emacspeak-eterm-keymap "r" 'emacspeak-eterm-search-backward)
    (define-key emacspeak-eterm-keymap "y"
      'emacspeak-eterm-kill-ring-save-region)
-   (define-key emacspeak-eterm-keymap "x" 'emacspeak-eterm-copy-region-to-register)
+   (define-key emacspeak-eterm-keymap "x"
+     'emacspeak-eterm-copy-region-to-register)
+(define-key emacspeak-eterm-keymap "v" 'emacspeak-eterm-paste-register)
    (define-key emacspeak-eterm-keymap "m" 'emacspeak-eterm-set-marker )
    (define-key emacspeak-eterm-keymap "\C-p"
      'emacspeak-eterm-toggle-pointer-mode)
@@ -590,15 +593,34 @@ emacspeak eterm pointer to a register."
   (declare (special emacspeak-eterm-marker 
                     emacspeak-eterm-pointer ))
   (copy-to-register register 
-   (marker-position emacspeak-eterm-marker)
-                  (marker-position emacspeak-eterm-pointer)
-                  nil)
+                    (marker-position emacspeak-eterm-marker)
+                    (marker-position emacspeak-eterm-pointer)
+                    nil)
   (emacspeak-auditory-icon 'mark-object)
   (message "Snarfed %s characters to register %c "
            (abs (- (marker-position emacspeak-eterm-marker )
                    (marker-position emacspeak-eterm-pointer)))
            register))
-(emacspeak-fix-interactive-command-if-necessary 'emacspeak-eterm-copy-region-to-register)
+(emacspeak-fix-interactive-command-if-necessary
+ 'emacspeak-eterm-copy-region-to-register)
+
+(defun emacspeak-eterm-paste-register (register)
+  "Paste contents of register at current location.
+If the specified register contains text, then that text is
+sent to the terminal as if it were typed by the user."
+  (interactive "*cRegister to paste: ")
+  (let ((contents (get-register register)))
+    (cond
+     ((stringp contents)
+      (term-send-raw-string contents)
+      (emacspeak-auditory-icon 'yank-object))
+     (t (error "Register %c does not contain text"
+               register)))))
+
+(emacspeak-fix-interactive-command-if-necessary 'emacspeak-eterm-paste-register)
+
+
+
 ;;}}}
 ;;{{{  Defining and speaking terminal windows:
 
@@ -913,6 +935,10 @@ activity in that area of the screen."
 "Single window please!"
 (delete-other-windows))
 
+(defadvice ansi-term (before emacspeak pre act)
+  "Single window please!"
+  (delete-other-windows))
+
 (defadvice term-mode   (after emacspeak pre act )
   "Customize eterm to work with Emacspeak.
 Additional commands provided by emacspeak under eterm are
@@ -1054,9 +1080,8 @@ See command emacspeak-toggle-eterm-autospeak bound to
         (delete-char  1)
         (dtk-tone 500 50))
        ((and (= new-row emacspeak-eterm-row )
-             (= 1 (- new-column emacspeak-eterm-column ))
-             (> new-end end ))          ;you inserted a character:
-        (if (= 32 (preceding-char ))
+             (= 1 (- new-column emacspeak-eterm-column ))) ;you inserted a character:
+        (if (= 32 last-input-event )
             (save-excursion
               (backward-char 2)
               (emacspeak-speak-word nil))
@@ -1065,7 +1090,10 @@ See command emacspeak-toggle-eterm-autospeak bound to
              (= 1 (abs(- new-column emacspeak-eterm-column ))))
         (emacspeak-speak-this-char (following-char )))
        ((= emacspeak-eterm-row new-row)
-        (emacspeak-speak-word ))
+        (if (= 32 (following-char))
+            (save-excursion (forward-char 1)
+                            (emacspeak-speak-word))        
+          (emacspeak-speak-word )))
        (t (emacspeak-speak-line nil )))
       (when (and (not  emacspeak-eterm-pointer-mode )
                  emacspeak-eterm-pointer)
@@ -1241,9 +1269,9 @@ completions for filename at point")))
 host. Hosts are added whenever a new hostname is encountered, and the
 list of known hostnames is persisted in file named by
 emacspeak-eterm-remote-hostnames")
-
+(declaim (special emacspeak-resource-directory))
 (defvar emacspeak-eterm-remote-hosts-cache
-  (expand-file-name "~/.emacspeak-hosts")
+  (concat emacspeak-resource-directory "/" ".hosts")
   "File where list of known remote hosts is cached")
 
 (defun emacspeak-eterm-load-remote-hosts-cache ()
