@@ -1,4 +1,5 @@
-;;; $Id: emacspeak-fix-interactive.el,v 7.0 1997/11/13 15:32:30 raman Exp $
+;;; emacspeak-fix-interactive.el --- Tools to make  Emacs' builtin prompts   speak
+;;; $Id: emacspeak-fix-interactive.el,v 8.0 1998/05/01 14:15:46 raman Exp $
 ;;; $Author: raman $ 
 ;;; Description: Fixes functions that use interactive to prompt for args.
 ;;; Approach suggested by hans@cs.buffalo.edu
@@ -8,14 +9,14 @@
 ;;; LCD Archive Entry:
 ;;; emacspeak| T. V. Raman |raman@adobe.com 
 ;;; A speech interface to Emacs |
-;;; $Date: 1997/11/13 15:32:30 $ |
-;;;  $Revision: 7.0 $ | 
+;;; $Date: 1998/05/01 14:15:46 $ |
+;;;  $Revision: 8.0 $ | 
 ;;; Location undetermined
 ;;;
 
 ;;}}}
 ;;{{{  Copyright:
-;;;Copyright (C) 1995, 1996, 1997  T. V. Raman  Adobe Systems Incorporated
+;;;Copyright (C) 1995, 1996, 1997, 1998   T. V. Raman  Adobe Systems Incorporated
 ;;; Copyright (c) 1994, 1995 by Digital Equipment Corporation.
 ;;; All Rights Reserved. 
 ;;;
@@ -141,26 +142,31 @@ its prompts. "
              (mapcar
               (function 
                (lambda (prompt)
-                 (` (let ((dtk-stop-immediately nil))
+                 (` (let
+                        ((dtk-stop-immediately nil)
+                         (emacspeak-speak-messages nil))
                       (dtk-speak
                        (,
-                        (format " %s "(or
-                         (if (= ?* (aref  prompt 0))
-                             (substring prompt 2 )
-                           (substring prompt 1 ))
-                         ""))))
+                        (format " %s "
+                                (or
+                                 (if (= ?* (aref  prompt 0))
+                                     (substring prompt 2 )
+                                   (substring prompt 1 ))
+                                 ""))))
                       ;; some interactive prompts guess defaults
                       (if (and
                            (or (= ?F  (aref (, prompt) 0))
                                (= ?f  (aref (, prompt) 0))
                                (= ?D  (aref  (, prompt) 0)))
                            default-directory)
-                          (dtk-speak default-directory ))
+                          (dtk-speak 
+                           (format " %s " default-directory )))
                       (call-interactively
                        '(lambda (&rest args)
                           (interactive (, prompt))
                           args) nil)))))
               interactive-list))))))))))
+
 ;;; inline function for use from other modules:
 
 (defsubst  emacspeak-fix-interactive-command-if-necessary (command)
