@@ -1,4 +1,5 @@
-;;; $Id: emacspeak-info.el,v 7.0 1997/11/13 15:32:30 raman Exp $
+;;; emacspeak-info.el --- Speech enable Info -- Emacs' online documentation viewer
+;;; $Id: emacspeak-info.el,v 8.0 1998/05/01 14:15:46 raman Exp $
 ;;; $Author: raman $ 
 ;;; Description:  Module for customizing Emacs info
 ;;; Keywords:emacspeak, audio interface to emacs
@@ -7,14 +8,14 @@
 ;;; LCD Archive Entry:
 ;;; emacspeak| T. V. Raman |raman@adobe.com 
 ;;; A speech interface to Emacs |
-;;; $Date: 1997/11/13 15:32:30 $ |
-;;;  $Revision: 7.0 $ | 
+;;; $Date: 1998/05/01 14:15:46 $ |
+;;;  $Revision: 8.0 $ | 
 ;;; Location undetermined
 ;;;
 
 ;;}}}
 ;;{{{  Copyright:
-;;;Copyright (C) 1995, 1996, 1997  T. V. Raman  Adobe Systems Incorporated
+;;;Copyright (C) 1995, 1996, 1997, 1998   T. V. Raman  Adobe Systems Incorporated
 ;;; Copyright (c) 1994, 1995 by Digital Equipment Corporation.
 ;;; All Rights Reserved. 
 ;;;
@@ -38,7 +39,7 @@
 (require 'cl)
 (declaim  (optimize  (safety 0) (speed 3)))
 (eval-when (compile)
-(require 'dtk-speak))
+  (require 'dtk-speak))
 (require 'emacspeak-sounds)
 (require 'emacspeak-speak)
 (require 'voice-lock)
@@ -62,6 +63,13 @@
 ;;}}}
 ;;{{{  Voices 
 
+(defvar Info-title-personality-alist
+  '((?* bold)
+    (?= bold-italic)
+    (?-  underline))
+  "*Alist of personality or list of personalities to use for pseudo-underlined titles.
+The alist key is the character the title is underlined with (?*, ?= or ?-).")
+  
 (dtk-define-voice 'info-node   "[:nh]")
 (dtk-define-voice 'info-xref "[:nu]")
 (dtk-define-voice 'info-menu-5 "[:nb]")
@@ -71,10 +79,11 @@
 
 ;;; Cloned from info.el Info-fontify-node 
 (defun Info-voiceify-node ()
-(declare (special Info-current-node))
+  (declare (special Info-title-personality-alist Info-current-node))
   (save-excursion
     (let ((buffer-read-only nil))
       (goto-char (point-min))
+      (setq voice-lock-mode t)
       (if (looking-at "^File: [^,: \t]+,?[ \t]+")
 	  (progn
 	    (goto-char (match-end 0))
@@ -82,15 +91,25 @@
 		(looking-at "[ \t]*[^:, \t\n]+:[ \t]+\\([^:,\t\n]+\\),?")
 	      (goto-char (match-end 0))
 	      (put-text-property (match-beginning 1) (match-end 1)
-				 'personality 'info-xref)
-	      )))
+				 'personality 'info-xref))))
+      (goto-char (point-min))
+      (while (re-search-forward "\n\\([^ \t\n].+\\)\n\\(\\*+\\|=+\\|-+\\)$"
+                                nil t)
+	(put-text-property (match-beginning 1) (match-end 1)
+			   'personality
+			   (cdr (assq (preceding-char) Info-title-personality-alist)))
+	;; This is a serious problem for trying to handle multiple
+	;; frame types at once.  We want this text to be invisible
+	;; on frames that can display the voice above.
+	(if (memq (framep (selected-frame)) '(x pc w32))
+	    (put-text-property (match-end 1) (match-end 2)
+			       'invisible t)))
       (goto-char (point-min))
       (while (re-search-forward "\\*Note[ \n\t]+\\([^:]*\\):" nil t)
 	(if (= (char-after (1- (match-beginning 0))) ?\") ; hack
 	    nil
 	  (put-text-property (match-beginning 1) (match-end 1)
-			     'personality 'info-xref)
-	  ))
+			     'personality 'info-xref)))
       (goto-char (point-min))
       (if (and (search-forward "\n* Menu:" nil t)
 	       (not (string-match "\\<Index\\>" Info-current-node))
@@ -99,13 +118,12 @@
 	  (let ((n 0))
 	    (while (re-search-forward "^\\* \\([^:\t\n]*\\):" nil t)
 	      (setq n (1+ n))
-	      (if (memq n '(5 9))   ; visual aids to help with 1-9 keys
+	      (if (memq n '(5 9))       ; visual aids to help with 1-9 keys
 		  (put-text-property (match-beginning 0)
 				     (1+ (match-beginning 0))
 				     'personality 'info-menu-5))
 	      (put-text-property (match-beginning 1) (match-end 1)
-				 'personality 'info-node)
-	      )))
+				 'personality 'info-node))))
       (set-buffer-modified-p nil))))
 
 ;;}}}
@@ -114,61 +132,61 @@
 ;;; Advice Info mode to voice lock 
 
 (defadvice Info-mode (after emacspeak pre act)
-"Set up voice locking if requested. 
+  "Set up voice locking if requested. 
 See variable `Info-voiceify`"
-(and   Info-voiceify (voice-lock-mode)))
+  (Info-voiceify-node))
 
 (defadvice Info-select-node (after emacspeak pre act)
   "Voiceify the Info node if requested."
   (let ((dtk-stop-immediately t ))
     (emacspeak-auditory-icon 'select-object)
     (and Info-voiceify (Info-voiceify-node))
-         (let ((start  (point ))
-               (window (get-buffer-window (current-buffer ))))
-           (save-excursion 
-             (forward-line (window-height window))
-             (emacspeak-speak-region start (point ))))))
+    (let ((start  (point ))
+          (window (get-buffer-window (current-buffer ))))
+      (save-excursion 
+        (forward-line (window-height window))
+        (emacspeak-speak-region start (point ))))))
 
 (defadvice info (after emacspeak pre act)
-"Cue user that info is up."
-(when (interactive-p)
-(emacspeak-auditory-icon 'help)
-(emacspeak-speak-line)))
+  "Cue user that info is up."
+  (when (interactive-p)
+    (emacspeak-auditory-icon 'help)
+    (emacspeak-speak-line)))
 
 
 (defadvice Info-scroll-up (after emacspeak pre act) 
-"Speak the screenful."
-(when (interactive-p)
-  (emacspeak-auditory-icon 'scroll)
-(let ((start  (point ))
-              (window (get-buffer-window (current-buffer ))))
-(save-excursion 
-          (forward-line (window-height window))
-          (emacspeak-speak-region start (point ))))))
+  "Speak the screenful."
+  (when (interactive-p)
+    (emacspeak-auditory-icon 'scroll)
+    (let ((start  (point ))
+          (window (get-buffer-window (current-buffer ))))
+      (save-excursion 
+        (forward-line (window-height window))
+        (emacspeak-speak-region start (point ))))))
 
 (defadvice Info-scroll-down (after emacspeak pre act) 
-"Speak the screenful."
-(when (interactive-p)
-  (emacspeak-auditory-icon 'scroll)
-(let ((start  (point ))
-              (window (get-buffer-window (current-buffer ))))
-(save-excursion 
-          (forward-line (window-height window))
-          (emacspeak-speak-region start (point ))))))
+  "Speak the screenful."
+  (when (interactive-p)
+    (emacspeak-auditory-icon 'scroll)
+    (let ((start  (point ))
+          (window (get-buffer-window (current-buffer ))))
+      (save-excursion 
+        (forward-line (window-height window))
+        (emacspeak-speak-region start (point ))))))
 
 (defadvice Info-exit (after emacspeak pre act)
-"Play an auditory icon to close info,
+  "Play an auditory icon to close info,
 and then cue the next selected buffer."
-(when (interactive-p )
-(dtk-stop)
-(emacspeak-auditory-icon 'close-object)
-(emacspeak-speak-mode-line)))
+  (when (interactive-p )
+    (dtk-stop)
+    (emacspeak-auditory-icon 'close-object)
+    (emacspeak-speak-mode-line)))
 
 
 (defadvice Info-next-reference (after emacspeak pre act)
-"Speak the line. "
-(when (interactive-p)
-(emacspeak-speak-line)))
+  "Speak the line. "
+  (when (interactive-p)
+    (emacspeak-speak-line)))
 
 (defadvice Info-prev-reference (after emacspeak pre act)
   "Speak the line. "
@@ -179,10 +197,10 @@ and then cue the next selected buffer."
 ;;{{{  Fix interactive commands where necessary:
 
 (mapcar 'emacspeak-fix-interactive-command-if-necessary 
-(list 'Info-search 
-'Info-index 
-'Info-goto-emacs-key-command-node 
-'Info-goto-emacs-command-node))
+        (list 'Info-search 
+              'Info-index 
+              'Info-goto-emacs-key-command-node 
+              'Info-goto-emacs-command-node))
 
 
 ;;}}}
