@@ -1,20 +1,21 @@
-;;; $Id: emacspeak-keymap.el,v 7.0 1997/11/13 15:32:30 raman Exp $
+;;; emacspeak-keymap.el --- Setup all keymaps and keybindings provided by Emacspeak
+;;; $Id: emacspeak-keymap.el,v 8.0 1998/05/01 14:15:46 raman Exp $
 ;;; $Author: raman $ 
 ;;; Description:  Module for setting up emacspeak keybindings
-;;; Keywords: Emacspeak, Keybinding, Spoken Output
+;;; Keywords: Emacspeak
 ;;{{{  LCD Archive entry: 
 
 ;;; LCD Archive Entry:
 ;;; emacspeak| T. V. Raman |raman@adobe.com 
 ;;; A speech interface to Emacs |
-;;; $Date: 1997/11/13 15:32:30 $ |
-;;;  $Revision: 7.0 $ | 
+;;; $Date: 1998/05/01 14:15:46 $ |
+;;;  $Revision: 8.0 $ | 
 ;;; Location undetermined
 ;;;
 
 ;;}}}
 ;;{{{  Copyright:
-;;;Copyright (C) 1995, 1996, 1997  T. V. Raman  Adobe Systems Incorporated
+;;;Copyright (C) 1995, 1996, 1997, 1998   T. V. Raman  Adobe Systems Incorporated
 ;;; Copyright (c) 1994, 1995 by Digital Equipment Corporation.
 ;;; All Rights Reserved. 
 ;;;
@@ -41,8 +42,12 @@
 (declaim  (optimize  (safety 0) (speed 3)))
 ;;{{{  Introduction:
 
+;;;Commentary:
+
 ;;; This module defines the emacspeak keybindings. 
 
+;;; Code:
+
 ;;}}}
 ;;{{{  variables: 
 
@@ -71,14 +76,19 @@
 ;;{{{  The actual bindings.
 
 (define-key help-map emacspeak-prefix 'emacspeak-describe-emacspeak)
-
+(define-key emacspeak-keymap '[left]
+  'emacspeak-speak-this-buffer-previous-display)
+(define-key emacspeak-keymap '[right]
+  'emacspeak-speak-this-buffer-next-display)
+(define-key emacspeak-keymap "/" 'emacspeak-speak-this-buffer-other-window-display)
+(define-key emacspeak-keymap '[down]
+  'emacspeak-read-next-line)
 (define-key emacspeak-keymap '[up]  'emacspeak-read-previous-line)
-(define-key emacspeak-keymap '[right]  'emacspeak-read-next-word)
-(define-key emacspeak-keymap '[left]  'emacspeak-read-previous-word)
-(define-key emacspeak-keymap '[down]  'emacspeak-read-next-line)
 (define-key emacspeak-keymap "x" 'emacspeak-view-register)
 (define-key emacspeak-keymap "w" 'emacspeak-speak-word)
+(define-key emacspeak-keymap "W" 'emacspeak-speak-spell-current-word)
 (define-key emacspeak-keymap "v" 'emacspeak-speak-version)
+(define-key emacspeak-keymap "\C-v" 'view-mode)
 (define-key emacspeak-keymap "t" 'emacspeak-speak-time )
 (define-key emacspeak-keymap "s" 'dtk-stop)
 (define-key emacspeak-keymap "r" 'emacspeak-speak-region)
@@ -99,7 +109,9 @@
 (define-key emacspeak-keymap "h" 'emacspeak-speak-help)
 (define-key emacspeak-keymap "j" 'emacspeak-hide-or-expose-block)
 (define-key emacspeak-keymap "\C-j" 'emacspeak-hide-speak-block-sans-prefix)
-(define-key emacspeak-keymap  "f" 'emacspeak-speak-current-field )
+(define-key emacspeak-keymap  "f"
+  'emacspeak-speak-buffer-filename )
+(define-key emacspeak-keymap  "\C-f" 'emacspeak-frame-label-or-switch-to-labelled-frame )
 (define-key emacspeak-keymap "c" 'emacspeak-speak-char)
 (define-key emacspeak-keymap "b" 'emacspeak-speak-buffer)
 (define-key emacspeak-keymap "a" 'emacspeak-speak-message-again )
@@ -110,7 +122,6 @@
 (define-key emacspeak-keymap "\C-h" 'emacspeak-learn-mode)
 (define-key emacspeak-keymap "\C-b" 'emacspeak-submit-bug )
 (define-key emacspeak-keymap "\"" 'emacspeak-speak-sexp-interactively)
-(define-key emacspeak-keymap "W" 'emacspeak-speak-word-interactively)
 (define-key emacspeak-keymap "p" 'dtk-pause)
 (define-key emacspeak-keymap "]" 'emacspeak-speak-page-interactively)
 (define-key emacspeak-keymap "[" 'emacspeak-speak-paragraph)
@@ -121,10 +132,13 @@
 (define-key emacspeak-keymap "B" 'emacspeak-speak-buffer-interactively)
 (define-key emacspeak-keymap "A" 'emacspeak-appt-repeat-announcement)
 (define-key emacspeak-keymap "?" 'search-backward )
-(define-key emacspeak-keymap "/" 'search-forward)
-(define-key emacspeak-keymap "." 'emacspeak-speak-browse-buffer )
-(define-key emacspeak-keymap "\C-c" 'emacspeak-speak-display-char)
+(define-key emacspeak-keymap "."
+  'emacspeak-speak-browse-buffer )
+(define-key emacspeak-keymap ";" 'emacspeak-realaudio )
+(define-key emacspeak-keymap "C" 'emacspeak-speak-display-char)
 (define-key emacspeak-keymap "\C-o" 'emacspeak-speak-other-window )
+(define-key emacspeak-keymap "\C-c" 'emacspeak-clipboard-copy)
+(define-key emacspeak-keymap "\C-y" 'emacspeak-clipboard-paste)
 (define-key emacspeak-keymap "\C-p" 'emacspeak-speak-previous-window)
 (define-key emacspeak-keymap "\C-n" 'emacspeak-speak-next-window )
 (define-key emacspeak-keymap ")" 'emacspeak-use-customized-blink-paren)
@@ -140,7 +154,6 @@
 (define-key emacspeak-keymap "\C-r" 'emacspeak-eterm-remote-term)
 (define-key emacspeak-keymap "\M-r"
   'emacspeak-remote-connect-to-server)
-(define-key emacspeak-keymap "\M-s" 'emacspeak-remote-startup-local-server)
 (define-key emacspeak-keymap "\M-d" 'emacspeak-pronounce-dispatch)
 (define-key emacspeak-keymap "\M-a" 'emacspeak-toggle-action-mode)
 (define-key emacspeak-keymap "\M-m" 'emacspeak-toggle-mail-alert)
@@ -158,7 +171,9 @@
     'emacspeak-speak-predefined-window ))
 
 
-
+(define-key emacspeak-keymap "N"
+  'emacspeak-view-emacspeak-news)
+(define-key emacspeak-keymap "D" 'emacspeak-view-emacspeak-doc)
 ;;; submap for setting dtk:
 (define-key emacspeak-dtk-submap "z" 'emacspeak-zap-dtk)
 (define-key emacspeak-dtk-submap "t" 'emacspeak-dial-dtk)
@@ -179,7 +194,7 @@
   'emacspeak-toggle-audio-indentation )
 (define-key emacspeak-dtk-submap "I" 'dtk-toggle-stop-immediately-while-typing )
 (define-key emacspeak-dtk-submap "f" 'dtk-set-character-scale)
-(define-key emacspeak-dtk-submap "d" 'dtk-select-driver)
+(define-key emacspeak-dtk-submap "d" 'dtk-select-server)
 (define-key emacspeak-dtk-submap "c" 'dtk-toggle-capitalization)
 (define-key emacspeak-dtk-submap "C" 'dtk-toggle-allcaps-beep)
 (define-key emacspeak-dtk-submap "b" 'dtk-toggle-debug)
@@ -191,11 +206,13 @@
     (format "%s" i )
     'dtk-set-predefined-speech-rate ))
 ;;; Put these in the global map:
-(global-set-key '[delete] 'dtk-toggle-punctuation-mode)
+(global-set-key '[C-left] 'emacspeak-previous-frame)
+(global-set-key '[C-right] 'emacspeak-next-frame)
 (global-set-key '[pause] 'dtk-stop)
 (or  (string-match  "Xemacs" emacs-version)
 (progn
-  (global-set-key '[C-down] 'emacspeak-read-line-by-line)
+  (global-set-key '[C-down] 'emacspeak-mark-forward-mark)
+  (global-set-key '[C-up] 'emacspeak-mark-backward-mark)
 (global-set-key '[S-up] 'emacspeak-skip-blank-lines-backward)
 (global-set-key '[S-down] 'emacspeak-skip-blank-lines-forward)
   (global-set-key '[27 up]  'emacspeak-owindow-previous-line)
@@ -210,7 +227,7 @@
   (global-set-key '[down] 'next-line)
   (global-set-key '[left] 'emacspeak-backward-char)
   (global-set-key '[right] 'emacspeak-forward-char)))
-
+(define-key help-map "\C-m" 'man)
 ;;}}}
 ;;{{{ Hacking minibuffer maps:
 
@@ -240,16 +257,15 @@ to do if you run emacspeak on a remote machine from inside a terminal
 that is running inside a local emacspeak session.  You can have the
 remote emacspeak use a different control key to give your fingers some
 relief."
-  (interactive
-   (list (read-key-sequence
-          "Press the key you would like to use as the emacspeak prefix")))
+  (interactive "kPress the key you would like to use as the emacspeak prefix")
   (declare (special emacspeak-prefix))
   (let ((current-use (lookup-key  global-map prefix-key)))
     (global-set-key prefix-key 'emacspeak-prefix-command)
     (unless (eq  current-use 'emacspeak-prefix-command)
-    (global-set-key (concat prefix-key prefix-key) current-use)
-    (message "Use %s %s to execute %s since %s is now the emacspeak prefix"
-             prefix-key prefix-key current-use prefix-key))))
+      (global-set-key (concat prefix-key prefix-key) current-use)
+      (message "Use %s %s to execute %s since %s is now the emacspeak prefix"
+               prefix-key prefix-key current-use
+               prefix-key))))
 
 ;;}}}
 (provide 'emacspeak-keymap)
