@@ -1,4 +1,5 @@
-;;; $Id: emacspeak-load-path.el,v 7.0 1997/11/13 15:32:30 raman Exp $
+;;; emacspeak-load-path.el --- Setup load-path correctly for Emacspeak compilation
+;;; $Id: emacspeak-load-path.el,v 8.0 1998/05/01 14:15:46 raman Exp $
 ;;; $Author: raman $ 
 ;;; Description:  Sets up load-path for emacspeak compilation and installation
 ;;; Keywords: Emacspeak, Speech extension for Emacs
@@ -6,14 +7,14 @@
 ;;; LCD Archive Entry:
 ;;; emacspeak| T. V. Raman |raman@adobe.com 
 ;;; A speech interface to Emacs |
-;;; $Date: 1997/11/13 15:32:30 $ |
-;;;  $Revision: 7.0 $ | 
+;;; $Date: 1998/05/01 14:15:46 $ |
+;;;  $Revision: 8.0 $ | 
 ;;; Location undetermined
 ;;;
 
 ;;}}}
 ;;{{{  Copyright:
-;;;Copyright (C) 1995, 1996, 1997  T. V. Raman  Adobe Systems Incorporated
+;;;Copyright (C) 1995, 1996, 1997, 1998   T. V. Raman  Adobe Systems Incorporated
 ;;; Copyright (c) 1994, 1995 by Digital Equipment Corporation.
 ;;; All Rights Reserved. 
 ;;;
@@ -35,11 +36,14 @@
 
 ;;}}}
 (defvar emacspeak-dir
-  (expand-file-name "/home/raman/emacs/lisp/emacspeak")
+  (expand-file-name "/user/raman/emacs/lisp/emacspeak")
   "Directory where emacspeak is installed. ")
 (or (member emacspeak-dir load-path )
 (setq load-path
       (cons emacspeak-dir 
                               load-path )))
 
+(defvar emacspeak-resource-directory (expand-file-name "~/.emacspeak")
+  "Directory where Emacspeak resource files such as pronunciation dictionaries are stored. ")
+
 (provide 'emacspeak-load-path)
