@@ -1,4 +1,5 @@
-;;; $Id: emacspeak-outline.el,v 7.0 1997/11/13 15:32:30 raman Exp $
+;;; emacspeak-outline.el --- Speech enable Outline --   Browsing  Structured Documents
+;;; $Id: emacspeak-outline.el,v 8.0 1998/05/01 14:15:46 raman Exp $
 ;;; $Author: raman $ 
 ;;; DescriptionEmacspeak extensions for outline-mode
 ;;; Keywords:emacspeak, audio interface to emacs Outlines
@@ -8,7 +9,7 @@
 ;;; emacspeak| T. V. Raman |raman@crl.dec.com 
 ;;; A speech interface to Emacs |
 ;;; $date: $ |
-;;;  $Revision: 7.0 $ | 
+;;;  $Revision: 8.0 $ | 
 ;;; Location undetermined
 ;;;
 
@@ -180,9 +181,9 @@
             (setq end (point)))
         (error (setq end (point-max)))))
     (and (y-or-n-p
-          (format  "Speak section %s  containing %s lines "
-                   (thing-at-point 'line)
-                   (count-lines start end )))
+          (format  "Speak %s lines from section %s"
+                   (count-lines start end )
+                   (thing-at-point 'line)))
          (emacspeak-speak-region start end ))))
 
 (defun emacspeak-outline-speak-next-heading ()
@@ -223,9 +224,9 @@ except that the outline section is optionally spoken"
             (setq end (point)))
         (error (setq end (point-max)))))
     (and (y-or-n-p
-          (format "Speak section %s containing %s lines"
-                  (thing-at-point 'line)
-                  (count-lines start end)))
+          (format "Speak %s lines from section %s"
+                  (count-lines start end)
+                  (thing-at-point 'line)))
          (emacspeak-speak-region start end ))))
 
 
@@ -258,7 +259,7 @@ except that the outline section is optionally spoken"
   "Provide auditory feedback about the child we zoomed into"
   (when (interactive-p)
     (emacspeak-auditory-icon 'open-object)
-    (message "Zoomed into outline %s  containing %s lines"
+    (message "Zoomed into outline %s containing %s lines"
              (thing-at-point 'line)
              (count-lines (point-min) (point-max)))))
 
@@ -269,6 +270,21 @@ except that the outline section is optionally spoken"
     (emacspeak-speak-line)))
 
 ;;}}}
+;;{{{ advise emacs 20's overlay oriented outliner to put
+;;properties we process
+
+(defadvice outline-flag-region (after emacspeak pre act comp)
+  "Set region to be inaudible if we just hid the region--
+make it audible otherwise. "
+  (ems-modify-buffer-safely
+   (if (ad-get-arg 2)                   ;hiding region
+       (put-text-property (ad-get-arg 0)
+                          (ad-get-arg 1)
+                          'invisible t)
+     (remove-text-properties (ad-get-arg 0)
+                             (ad-get-arg 1)
+                             '(invisible t)))))
+;;}}}
 (provide  'emacspeak-outline)
 ;;{{{  emacs local variables 
 
