@@ -1,4 +1,5 @@
-;;; $Id: emacspeak-pronounce.el,v 7.0 1997/11/13 15:32:30 raman Exp $
+;;; emacspeak-pronounce.el --- Implements Emacspeak pronunciation dictionaries
+;;; $Id: emacspeak-pronounce.el,v 8.0 1998/05/01 14:15:46 raman Exp $
 ;;; $Author: raman $ 
 ;;; Description: Emacspeak pronunciation dictionaries
 ;;; Keywords:emacspeak, audio interface to emacs customized pronunciation
@@ -7,14 +8,14 @@
 ;;; LCD Archive Entry:
 ;;; emacspeak| T. V. Raman |raman@adobe.com
 ;;; A speech interface to Emacs |
-;;; $Date: 1997/11/13 15:32:30 $ |
-;;;  $Revision: 7.0 $ | 
+;;; $Date: 1998/05/01 14:15:46 $ |
+;;;  $Revision: 8.0 $ | 
 ;;; Location undetermined
 ;;;
 
 ;;}}}
 ;;{{{  Copyright:
-;;;Copyright (C) 1995, 1996, 1997  T. V. Raman  Adobe Systems Incorporated
+;;;Copyright (C) 1995, 1996, 1997, 1998   T. V. Raman  Adobe Systems Incorporated
 ;;; Copyright (c) 1995 by T. V. Raman Adobe Systems Incorporated 
 ;;; All Rights Reserved. 
 ;;;
@@ -41,6 +42,9 @@
 (require 'thingatpt)
 (require 'emacspeak-sounds)
 (eval-when (compile)
+                                        ;avoid recursive include during compile
+
+  (provide 'emacspeak-pronounce)
   (require 'dtk-speak))
 ;;{{{  Introduction
 
@@ -187,6 +191,11 @@ modes."
 ;;}}}
 ;;{{{ defining some inheritance relations:
 
+;;; gnus server mode inherits from gnus group mode
+
+(emacspeak-pronounce-add-super  'gnus-group-mode
+                                'gnus-server-mode)
+
 ;;; c++ mode inherits from C mode
 (emacspeak-pronounce-add-super  'c-mode 'c++-mode)
 
@@ -249,7 +258,12 @@ buffer. Modifies text in buffer. "
 "File that holds the persistent emacspeak pronunciation
 dictionaries.")
 
-(setq emacspeak-pronounce-dictionaries-file (expand-file-name "~/.emacspeak-dictionary"))
+
+  (declaim (special emacspeak-resource-directory))
+  (setq emacspeak-pronounce-dictionaries-file
+        (concat emacspeak-resource-directory "/"
+                ".dictionary"))
+
 
 (defun emacspeak-pronounce-save-dictionaries  ()
   "Writes out the persistent emacspeak pronunciation dictionaries."
@@ -391,8 +405,7 @@ First loads any persistent dictionaries if not already loaded. "
       (emacspeak-pronounce-load-dictionaries))
     (unless  (string= key-type  "buffer")
       (emacspeak-pronounce-add-dictionary-entry key word pronunciation)
-      (when (y-or-n-p "Refresh pronunciations in this buffer")
-        (funcall 'emacspeak-pronounce-refresh-pronunciations)))
+        (funcall 'emacspeak-pronounce-refresh-pronunciations))
     (when (string= key-type  "buffer")
       (emacspeak-pronounce-add-buffer-local-dictionary-entry  word pronunciation))))
 
