@@ -1,4 +1,5 @@
-;;; $Id: emacspeak-redefine.el,v 7.0 1997/11/13 15:32:30 raman Exp $
+;;; emacspeak-redefine.el --- Redefines some key Emacs builtins to speak
+;;; $Id: emacspeak-redefine.el,v 8.0 1998/05/01 14:15:46 raman Exp $
 ;;; $Author: raman $ 
 ;;; Description:  Emacspeak's redefinition of some key functions.
 ;;; Emacspeak does most of its work by advising other functions to speak.
@@ -9,14 +10,14 @@
 ;;; LCD Archive Entry:
 ;;; emacspeak| T. V. Raman |raman@adobe.com 
 ;;; A speech interface to Emacs |
-;;; $Date: 1997/11/13 15:32:30 $ |
-;;;  $Revision: 7.0 $ | 
+;;; $Date: 1998/05/01 14:15:46 $ |
+;;;  $Revision: 8.0 $ | 
 ;;; Location undetermined
 ;;;
 
 ;;}}}
 ;;{{{  Copyright:
-;;;Copyright (C) 1995, 1996, 1997  T. V. Raman  Adobe Systems Incorporated
+;;;Copyright (C) 1995, 1996, 1997, 1998   T. V. Raman  Adobe Systems Incorporated
 ;;; Copyright (c) 1994, 1995 by Digital Equipment Corporation.
 ;;; All Rights Reserved. 
 ;;;
@@ -80,7 +81,7 @@
 
 (defun emacspeak-self-insert-command (arg)
   "Insert a character.
-Speaks the character if \\{emacspeak-character-echo} is true.
+Speaks the character if emacspeak-character-echo is true.
 See  command emacspeak-toggle-word-echo bound to
 \\[emacspeak-toggle-word-echo].
 Toggle variable dtk-stop-immediately-while-typing if you want to have
