@@ -1,4 +1,5 @@
-;;; $Id: emacspeak-setup.el,v 7.0 1997/11/13 15:32:30 raman Exp $
+;;; emacspeak-setup.el --- Setup Emacspeak environment  --loaded to start Emacspeak
+;;; $Id: emacspeak-setup.el,v 8.0 1998/05/01 14:15:46 raman Exp $
 ;;; $Author: raman $ 
 ;;; Description:  File for setting up and starting Emacspeak
 ;;; Keywords: Emacspeak, Setup, Spoken Output
@@ -6,14 +7,14 @@
 ;;; LCD Archive Entry:
 ;;; emacspeak| T. V. Raman |raman@adobe.com 
 ;;; A speech interface to Emacs |
-;;; $Date: 1997/11/13 15:32:30 $ |
-;;;  $Revision: 7.0 $ | 
+;;; $Date: 1998/05/01 14:15:46 $ |
+;;;  $Revision: 8.0 $ | 
 ;;; Location undetermined
 ;;;
 
 ;;}}}
 ;;{{{  Copyright:
-;;;Copyright (C) 1995, 1996, 1997  T. V. Raman  Adobe Systems Incorporated
+;;;Copyright (C) 1995, 1996, 1997, 1998   T. V. Raman  Adobe Systems Incorporated
 ;;; Copyright (c) 1994, 1995 by Digital Equipment Corporation.
 ;;; All Rights Reserved. 
 ;;;
@@ -39,7 +40,7 @@
 (eval-when (compile)
 (require 'emacspeak-speak))
 (defvar emacspeak-dir
-  (expand-file-name  "/home/raman/emacs/lisp/emacspeak")
+  (expand-file-name  "/user/raman/emacs/lisp/emacspeak")
   "Directory where emacspeak is installed. ")
 
   (defvar emacspeak-sounds-directory
@@ -49,7 +50,7 @@
             "Directory containing auditory icons for Emacspeak.")
 
 (defvar emacspeak-play-program
-  (or (getenv "PLAY_PROGRAM")
+  (or (getenv "EMACSPEAK_PLAY_PROGRAM")
 (concat emacspeak-dir "/play"))
 "Name of executable that plays sound files. ")
 
@@ -67,7 +68,7 @@
 )
 
 (defvar dtk-default-speech-rate 225
-"Default speech rate at which Dectalk is started. ")
+"*Default speech rate at which Dectalk is started. ")
 
 (add-hook 'dtk-startup-hook 
 (function (lambda () 
@@ -76,21 +77,30 @@
 (setq emacspeak-startup-hook nil )
 ;;; Use (add-hook 'emacspeak-startup-hook ...)
 ;;; to add your personal settings. 
-
+(defvar emacspeak-resource-directory (expand-file-name "~/.emacspeak")
+  "Directory where Emacspeak resource files such as pronunciation dictionaries are stored. ")
 (emacspeak)
 
-                  ;;; turn on automatic voice locking for modes
+                  ;;; turn on automatic voice locking , split caps and punctuations
+;;; for programming modes
  (mapcar
   (function (lambda (hook)
               (add-hook hook
                         (function (lambda ()
-                                    (setq dtk-punctuation-mode "all")
-(emacspeak-dtk-sync))))))
-  (list 'c-mode-hook
- 'lisp-mode-hook
- 'emacs-lisp-mode-hook
-'lisp-interaction-mode-hook
- 'perl-mode-hook
- 'text-mode-hook
- 'tcl-mode-hook
- 'dired-mode-hook))
+                                    (voice-lock-mode 1)
+                                    (dtk-set-punctuations "all")
+                                    (or dtk-split-caps
+                                        (dtk-toggle-split-caps))
+                                    (or emacspeak-audio-indentation
+                                        (emacspeak-toggle-audio-indentation))
+                                    (emacspeak-dtk-sync))))))
+  (list 'c-mode-common-hook
+        'py-mode-hook
+        'lisp-mode-hook
+        'emacs-lisp-mode-hook
+        'lisp-interaction-mode-hook
+        'perl-mode-hook
+        'tex-mode-hook
+        'tcl-mode-hook
+        'html-helper-mode-hook
+        'dired-mode-hook))
