@@ -1,4 +1,5 @@
-;;; $Id: emacspeak-sounds.el,v 7.0 1997/11/13 15:32:30 raman Exp $
+;;; emacspeak-sounds.el --- Defines Emacspeak auditory icons
+;;; $Id: emacspeak-sounds.el,v 8.0 1998/05/01 14:15:46 raman Exp $
 ;;; $Author: raman $ 
 ;;; Description:  Module for adding sound cues to emacspeak
 ;;; Keywords:emacspeak, audio interface to emacs, auditory icons 
@@ -7,14 +8,14 @@
 ;;; LCD Archive Entry:
 ;;; emacspeak| T. V. Raman |raman@adobe.com 
 ;;; A speech interface to Emacs |
-;;; $Date: 1997/11/13 15:32:30 $ |
-;;;  $Revision: 7.0 $ | 
+;;; $Date: 1998/05/01 14:15:46 $ |
+;;;  $Revision: 8.0 $ | 
 ;;; Location undetermined
 ;;;
 
 ;;}}}
 ;;{{{  Copyright:
-;;;Copyright (C) 1995, 1996, 1997  T. V. Raman  Adobe Systems Incorporated
+;;;Copyright (C) 1995, 1996, 1997, 1998   T. V. Raman  Adobe Systems Incorporated
 ;;; Copyright (c) 1994, 1995 by Digital Equipment Corporation.
 ;;; All Rights Reserved. 
 ;;;
@@ -114,6 +115,11 @@ file-name is the name of the sound file that produces this icon."
 (emacspeak-define-sound 'modified-object "boing8.au")
 (emacspeak-define-sound 'unmodified-object "drop_of_water_beep.au")
 (emacspeak-define-sound 'mark-object "beep_8.au")
+;;; we need better sounds for left, right and center justification
+(emacspeak-define-sound 'center "center.au")
+(emacspeak-define-sound 'right "right.au")
+(emacspeak-define-sound 'left "left.au")
+(emacspeak-define-sound 'full "drop_of_water_beep.au")
 (emacspeak-define-sound 'fill-object "spring_beep.au")
 (emacspeak-define-sound 'select-object "pure_saw_beep.au")
 (emacspeak-define-sound 'button "button.au")
