@@ -1,20 +1,21 @@
-;;; $Id: emacspeak-speak.el,v 7.0 1997/11/13 15:32:30 raman Exp $
+;;; emacspeak-speak.el --- Implements Emacspeak's core speech services
+;;; $Id: emacspeak-speak.el,v 8.0 1998/05/01 14:15:46 raman Exp $
 ;;; $Author: raman $ 
 ;;; Description:  Contains the functions for speaking various chunks of text
-;;; Keywords: Emacspeak, Speak, Spoken Output
+;;; Keywords: Emacspeak,  Spoken Output
 ;;{{{  LCD Archive entry: 
 
 ;;; LCD Archive Entry:
 ;;; emacspeak| T. V. Raman |raman@adobe.com 
 ;;; A speech interface to Emacs |
-;;; $Date: 1997/11/13 15:32:30 $ |
-;;;  $Revision: 7.0 $ | 
+;;; $Date: 1998/05/01 14:15:46 $ |
+;;;  $Revision: 8.0 $ | 
 ;;; Location undetermined
 ;;;
 
 ;;}}}
 ;;{{{  Copyright:
-;;;Copyright (C) 1995, 1996, 1997  T. V. Raman  Adobe Systems Incorporated
+;;;Copyright (C) 1995, 1996, 1997, 1998   T. V. Raman  Adobe Systems Incorporated
 ;;; Copyright (c) 1994, 1995 by Digital Equipment Corporation.
 ;;; All Rights Reserved. 
 ;;;
@@ -50,10 +51,13 @@
 ;;}}}
 ;;{{{  Introduction:
 
-;;; This module defines the core output functions used by emacspeak.
-;;; It depends on the dtk-speak module.
+;;; Commentary:
+
+;;; This module defines the core speech services used by emacspeak.
+;;; It depends on the speech server interface modules
 ;;; It protects other parts of emacspeak
-;;; from becoming dependent on the dtk module.
+;;; from becoming dependent on the speech server modules
+;;; Code:
 
 ;;}}}
 ;;{{{  Macros
@@ -366,30 +370,38 @@ current local  value to the result. "
 ;;{{{  indentation:
 
 (defvar emacspeak-audio-indentation nil
-  "If non-nil , then speaking a line indicates its indentation.
- Do not set this by hand, use command
-emacspeak-toggle-audio-indentation bound to
-\\[emacspeak-toggle-audio-indentation]. ")
+  "If non-nil , then speaking a line indicates its indentation.  Do not
+set this by hand, use command emacspeak-toggle-audio-indentation bound
+to \\[emacspeak-toggle-audio-indentation]. ")
+
+(make-variable-buffer-local 'emacspeak-audio-indentation)
 
 ;;; Indicate indentation.
- ;;; Argument indent   indicates number of columns to indent. 
+;;; Argument indent   indicates number of columns to indent. 
+
 (defsubst emacspeak-indent (indent)
   (when (> indent 1 )
     (let ((duration (+ 50 (* 20  indent ))))
       (dtk-tone  250 duration))))
-    
 
 (defvar emacspeak-audio-indentation-methods
   '(("speak" . "speak")
     ("tone" . "tone"))
   "Possible methods of indicating indentation. ")
+(defvar emacspeak-audio-indentation-method "speak"
+  "*Current technique used to cue indentation
+See variable emacspeak-audio-indentation-methods for
+possible values.
+Automatically becomes local in any buffer where it is set.")
 
+(make-variable-buffer-local
+ 'emacspeak-audio-indentation-method)
 
 (defun emacspeak-toggle-audio-indentation (&optional prefix)
   "Toggle state of  Emacspeak  audio indentation.
 Interactive prefix arg means toggle  the global default value, and then set the
 current local  value to the result.
- Specifying the method of indentation as `tones'
+Specifying the method of indentation as `tones'
 results in the Dectalk producing a tone whose length is a function of the
 line's indentation. Specifying `speak'
 results in the number of initial spaces being spoken."
@@ -401,14 +413,10 @@ results in the number of initial spaces being spoken."
     (setq-default  emacspeak-audio-indentation
                    (not  (default-value 'emacspeak-audio-indentation )))
     (setq emacspeak-audio-indentation (default-value 'emacspeak-audio-indentation )))
-   (t (make-local-variable'emacspeak-audio-indentation)
-      (setq emacspeak-audio-indentation 
+   (t (setq emacspeak-audio-indentation 
 	    (not emacspeak-audio-indentation ))))
   (when emacspeak-audio-indentation
-    (setq emacspeak-audio-indentation
-          (completing-read  "What kind of audio indentation would you like? "
-                            emacspeak-audio-indentation-methods  nil t
-                            "speak" ))
+    (setq emacspeak-audio-indentation emacspeak-audio-indentation-method)
     (and prefix
          (setq-default emacspeak-audio-indentation
                        emacspeak-audio-indentation )))
@@ -423,23 +431,7 @@ results in the number of initial spaces being spoken."
 
 (defsubst   emacspeak-dtk-sync ()
   "Bring emacspeak and dtk in sync. "
-  (declare (special dtk-speaker-process
-                    dtk-punctuation-mode dtk-speech-rate dtk-character-scale
-                    dtk-capitalize dtk-split-caps
-                    dtk-allcaps-beep))
-  (let ((command (concat 
-                  (format "dectalk_set_punctuations %s  \n "
-                          dtk-punctuation-mode )
-                  (format "dectalk_capitalize %s \n "
-                          (if dtk-capitalize 1  0 ))
-                  (format "dectalk_allcaps_beep %s \n "
-                          (if dtk-allcaps-beep 1  0 ))
-                  (format "dectalk_split_caps  %s \n "
-                          (if dtk-split-caps 1 0 ))
-                  (format "dectalk_set_speech_rate %s \n" dtk-speech-rate)
-                  (format "dectalk_set_character_scale %s \n "
-                          dtk-character-scale))))
-    (process-send-string dtk-speaker-process command )))
+  (dtk-interp-sync))
 
 ;;}}}
 ;;{{{ functions: 
@@ -507,6 +499,7 @@ emacspeak-hide-or-expose-block are indicated with auditory icon ellipses."
           (dtk-stop-immediately dtk-stop-immediately)
           (inhibit-point-motion-hooks t)
           (line nil)
+          (auditory-icon nil)
           (orig (point)) 
           (indent nil))
       (beginning-of-line)
@@ -572,15 +565,53 @@ emacspeak-hide-or-expose-block are indicated with auditory icon ellipses."
               (setq indent (format "indent %d" indent))
               (put-text-property   0 (length indent)
                                    'personality 'indent-voice  indent )
+              (setq auditory-icon (get-text-property 0  'auditory-icon line))
+              (when auditory-icon
+                (put-text-property   0 (length indent)
+                                     'auditory-icon auditory-icon  indent ))
               (dtk-speak (concat indent line)))
           (dtk-speak line)))))))
 
+(defvar emacspeak-speak-last-spoken-word-position nil
+  "Records position of the last word that was spoken.
+Local to each buffer. Used to decide if we should spell the word
+rather than speak it.")
+
+(make-variable-buffer-local 'emacspeak-speak-last-spoken-word-position)
+(defsubst emacspeak-speak-spell-word (word)
+  "Spell word "
+  (declare (special voice-lock-mode))
+  (let ((result "")
+        (char-string ""))
+    (loop for char across word
+          do
+          (setq char-string (format "%c " char))
+          (when (and (<= ?A char)
+                     (<= char ?Z))
+            (if voice-lock-mode
+            (put-text-property 0 1
+                               'personality 'paul-animated
+                               char-string)
+            (setq char-string (format "cap %s " char-string))))
+          (setq result
+                (concat result
+                        char-string)))
+    (dtk-speak result)))
+
+(defun emacspeak-speak-spell-current-word ()
+  "Spell word at  point"
+  (interactive)
+  (emacspeak-speak-spell-word (word-at-point)))
+  
 (defun emacspeak-speak-word (&optional arg) 
   "Speak current word.
   With prefix arg, speaks the rest of the word from point.
-Negative prefix arg speaks from start of word to point."
+Negative prefix arg speaks from start of word to point.
+If executed  on the same buffer position a second time, the word is
+spelt instead of being spoken."
   (interactive "P")
-  (declare (special voice-lock-mode))
+  (declare (special voice-lock-mode
+                    emacspeak-speak-last-spoken-word-position))
   (when (listp arg) (setq arg (car arg )))
   (emacspeak-handle-action-at-point)
   (cond
@@ -589,7 +620,8 @@ Negative prefix arg speaks from start of word to point."
         (let ((orig (point))
               (inhibit-point-motion-hooks t)
               (start nil)
-              (end nil))
+              (end nil)
+              (speaker 'dtk-speak))
           (forward-word 1)
           (setq end (point))
           (backward-word 1)
@@ -598,7 +630,14 @@ Negative prefix arg speaks from start of word to point."
            ((null arg ))
            ((> arg 0) (setq start orig))
            ((< arg 0) (setq end orig )))
-          (dtk-speak  (buffer-substring  start end )))))))
+          ;; select speak or spell
+          (cond
+           ((and (interactive-p)
+                 (eq emacspeak-speak-last-spoken-word-position orig))
+            (setq speaker 'emacspeak-speak-spell-word)
+            (setq emacspeak-speak-last-spoken-word-position nil))
+           (t (setq  emacspeak-speak-last-spoken-word-position orig)))
+          (funcall speaker  (buffer-substring  start end )))))))
 
 (defsubst emacspeak-is-alpha-p (c) (= 119 (char-syntax c)))
 
@@ -942,48 +981,47 @@ Negative prefix arg speaks from start of buffer to point."
     (save-excursion
       (set-buffer minibuff)
       (emacspeak-speak-buffer arg))))
-(or (string-match "19.30" emacs-version)
-    (progn 
-(defun next-completion (n)
-  "Move to the next item in the completion list.
+(unless (fboundp 'next-completion)
+  (progn 
+    (defun next-completion (n)
+      "Move to the next item in the completion list.
 WIth prefix argument N, move N items (negative N means move backward)."
-  (interactive "p")
-  (while (and (> n 0) (not (eobp)))
-    (let ((prop (get-text-property (point) 'mouse-face))
-	  (end (point-max)))
-      ;; If in a completion, move to the end of it.
-      (if prop
-	  (goto-char (next-single-property-change (point) 'mouse-face nil end)))
-      ;; Move to start of next one.
-      (goto-char (next-single-property-change (point) 'mouse-face nil end)))
-    (setq n (1- n)))
-  )
+      (interactive "p")
+      (while (and (> n 0) (not (eobp)))
+        (let ((prop (get-text-property (point) 'mouse-face))
+              (end (point-max)))
+          ;; If in a completion, move to the end of it.
+          (if prop
+              (goto-char (next-single-property-change (point) 'mouse-face nil end)))
+          ;; Move to start of next one.
+          (goto-char (next-single-property-change (point) 'mouse-face nil end)))
+        (setq n (1- n)))
+      )
 
-(defun previous-completion (n)
-  "Move to the previous item in the completion list."
-  (interactive "p")
-  (setq n (- n ))
-  (while (and (< n 0) (not (bobp)))
-    (let ((prop (get-text-property (1- (point)) 'mouse-face))
-	  (end (point-min)))
-      ;; If in a completion, move to the start of it.
-      (if prop
-	  (goto-char (previous-single-property-change
-		      (point) 'mouse-face nil end)))
-      ;; Move to end of the previous completion.
-      (goto-char (previous-single-property-change (point) 'mouse-face nil end))
-      ;; Move to the start of that one.
-      (goto-char (previous-single-property-change (point) 'mouse-face nil end)))
-    (setq n (1+ n))))
+    (defun previous-completion (n)
+      "Move to the previous item in the completion list."
+      (interactive "p")
+      (setq n (- n ))
+      (while (and (< n 0) (not (bobp)))
+        (let ((prop (get-text-property (1- (point)) 'mouse-face))
+              (end (point-min)))
+          ;; If in a completion, move to the start of it.
+          (if prop
+              (goto-char (previous-single-property-change
+                          (point) 'mouse-face nil end)))
+          ;; Move to end of the previous completion.
+          (goto-char (previous-single-property-change (point) 'mouse-face nil end))
+          ;; Move to the start of that one.
+          (goto-char (previous-single-property-change (point) 'mouse-face nil end)))
+        (setq n (1+ n))))
 
 
-(declaim (special completion-list-mode-map))
-(or completion-list-mode-map
-    (make-sparse-keymap ))
-(define-key completion-list-mode-map '[right] 'next-completion)
-(define-key completion-list-mode-map '[left] 'previous-completion)
-
-)) ;; end emacs pre-19.30 specials 
+    (declaim (special completion-list-mode-map))
+    (or completion-list-mode-map
+        (make-sparse-keymap ))
+    (define-key completion-list-mode-map '[right] 'next-completion)
+    (define-key completion-list-mode-map '[left] 'previous-completion)
+    ));; end emacs pre-19.30 specials 
 
 
 
@@ -1082,7 +1120,6 @@ indicating the arrival  of new mail when displaying the mode line."
 ;;}}}
 ;;{{{  Speak mode line information
 
-
 ;;;compute current line number 
 (defsubst emacspeak-get-current-line-number()
   (let ((start (point)))
@@ -1112,7 +1149,16 @@ indicating the arrival  of new mail when displaying the mode line."
   (dtk-stop)
   (emacspeak-dtk-sync)
   (force-mode-line-update)
-  (let ((dtk-stop-immediately nil )) 
+  (let ((dtk-stop-immediately nil )
+        (frame-info nil))
+    (cond
+     ((> (length (frame-list)) 1)
+      (setq frame-info
+            (or (frame-parameter (selected-frame) 'emacspeak-label)
+                (format "Frame %s " (frame-parameter (selected-frame) 'name))))
+      (put-text-property 0 (length frame-info)
+                         'personality 'annotation-voice frame-info))
+     (t (setq frame-info "")))
     (when (buffer-modified-p )
       (dtk-tone 700 70))
     (when buffer-read-only
@@ -1121,20 +1167,19 @@ indicating the arrival  of new mail when displaying the mode line."
       (and (emacspeak-mail-alert-user)
            (dtk-tone 450 75)))
     (dtk-speak
-     (format  "%s %s %s  %d%%  %s"
-              (if line-number-mode
-                  (format "line %d"
-                          (emacspeak-get-current-line-number))
-                "")
-              (if column-number-mode
-                  (format "Column %d"
-                          (current-column))
-                          "")
-              (if  (buffer-file-name ) 
-                  (file-name-nondirectory  (buffer-file-name ))
-                (buffer-name ))
-              (emacspeak-get-current-percentage-into-buffer)
-              (if  major-mode major-mode "")))))
+     (concat frame-info 
+             (format  "%s %s %s  %d%%  %s"
+                      (if line-number-mode
+                          (format "line %d"
+                                  (emacspeak-get-current-line-number))
+                        "")
+                      (if column-number-mode
+                          (format "Column %d"
+                                  (current-column))
+                        "")
+                      (buffer-name)
+                      (emacspeak-get-current-percentage-into-buffer)
+                      (if  major-mode major-mode ""))))))
 
 (defun emacspeak-speak-minor-mode-line ()
   "Speak the minor mode-information. "
@@ -1160,6 +1205,15 @@ indicating the arrival  of new mail when displaying the mode line."
 (interactive)
 (message "line %d"
          (emacspeak-get-current-line-number)))
+(defun emacspeak-speak-buffer-filename ()
+  "Speak name of file being visited in current buffer.
+Speak default directory if invoked in a dired buffer,
+or when the buffer is not visiting any file."
+  (interactive)
+  (dtk-speak
+   (or (buffer-file-name)
+       (format "Default directory is %s"
+               default-directory))))
 
 ;;}}}
 ;;{{{  Speak text without moving point
@@ -1215,10 +1269,24 @@ Default is to read the next word. "
 ;;}}}
 ;;{{{  Speak misc information e.g. time, version, current-kill  etc 
 
+(defvar emacspeak-speak-time-format-string 
+"%_I %M %p on %A, %B %_e, %Y"
+"*Format string that specifies how the time should be
+spoken.  See the documentation for function
+format-time-string")
+
 (defun emacspeak-speak-time ()
   "Speak the time. "
   (interactive)
-  (dtk-speak (current-time-string )))
+  (declare (special emacspeak-speak-time-format-string
+                    emacspeak-speak-messages))
+  (save-excursion
+    (set-buffer (get-buffer-create " *dtk-scratch-buffer* "))
+    (let ((emacspeak-speak-messages nil))
+    (unless (eq  dtk-punctuation-mode "some")
+      (dtk-set-punctuations "some"))
+    (dtk-speak (format-time-string emacspeak-speak-time-format-string)))))
+
 
 (defun emacspeak-speak-version ()
   "Announce version information for running emacspeak. "
@@ -1400,29 +1468,59 @@ Pressing  C-g breaks out, leaving point on last chunk that was spoken.
 (defvar emacspeak-read-by-line-by-line-tick 1.0
   "Granularity of time for reading line-by-line. ")
 
-(defun emacspeak-read-line-by-line ()
-  "Read line by line until interrupted"
+;(defun emacspeak-read-line-by-line ()
+;  "Read line by line until interrupted"
+;  (interactive)
+;  (let ((count 0)
+;        (line-length 0)
+;        (continue t))
+;    (while 
+;        (and continue
+;             (not (eobp)))
+;      (setq dtk-last-output "")
+;      (call-interactively 'next-line)
+;      (setq line-length (length  (thing-at-point 'line)))
+;      (setq count 0)
+;      (when (> line-length 0)
+;        (while(and (< count
+;                      (1+ (/ line-length emacspeak-read-line-by-line-quotient)))
+;                   (setq continue
+;                         (sit-for
+;                          emacspeak-read-by-line-by-line-tick 0 nil ))
+;                   (not (string-match  "done" dtk-last-output))
+;                   (incf count))))))
+;  (emacspeak-auditory-icon 'task-done)
+;  (message "done moving "))
+
+;;}}}
+;;{{{  skimming 
+(defun emacspeak-speak-skim-paragraph()
+  "Skim paragraph.
+Skimming a paragraph results in the speech speeding up after
+the first clause.
+Speech is scaled by the value of dtk-speak-skim-scale"
   (interactive)
-  (let ((count 0)
-        (line-length 0)
-        (continue t))
-    (while 
-        (and continue
-             (not (eobp)))
-      (setq dtk-last-output "")
-      (call-interactively 'next-line)
-      (setq line-length (length  (thing-at-point 'line)))
-      (setq count 0)
-      (when (> line-length 0)
-        (while(and (< count
-                      (1+ (/ line-length emacspeak-read-line-by-line-quotient)))
-                   (setq continue
-                         (sit-for
-                          emacspeak-read-by-line-by-line-tick 0 nil ))
-                   (not (string-match  "done" dtk-last-output))
-                   (incf count))))))
-  (emacspeak-auditory-icon 'task-done)
-  (message "done moving "))
+  (save-excursion 
+    (let ((inhibit-point-motion-hooks t)
+          (start nil)
+          (end nil))
+      (forward-paragraph 1) 
+      (setq end (point))
+      (backward-paragraph 1)
+      (setq start (point))
+      (dtk-speak (buffer-substring  start end )
+                 'skim))))
+
+(defun emacspeak-speak-skim-next-paragraph()
+  "Skim next paragraph."
+  (interactive)
+  (forward-paragraph 1)
+  (emacspeak-speak-skim-paragraph))
+
+(defun emacspeak-speak-skim-buffer ()
+  "Skim the current buffer  a paragraph at a time."
+  (interactive)
+  (emacspeak-execute-repeatedly 'emacspeak-speak-skim-next-paragraph))
 
 ;;}}}
 ;;{{{  Learn mode 
@@ -1572,12 +1670,7 @@ Will be improved if it proves useful."
 (defun emacspeak-announce (announcement)
   "Speak the announcement, if possible.
 Otherwise just display a message. "
-  (declare (special dtk-speaker-process ))
-  (if (and (featurep 'dtk-speak)
-           dtk-speaker-process 
-           (eq 'run  (process-status dtk-speaker-process )))
-      (dtk-speak announcement )
-    (message announcement)))
+  (message announcement))
 
 ;;}}}
 ;;{{{  Using emacs's windows usefully:
@@ -2172,22 +2265,29 @@ Warning! Contents of file filename will be overwritten."
     (save-excursion
       (set-buffer buffer)
       (erase-buffer)
+      (insert "DOC --- Automatically generated by command emacspeak-generate-documentation\n\$Id: emacspeak-speak.el,v 8.0 1998/05/01 14:15:46 raman Exp $\n")
       (mapcar 
        (function
         (lambda (f)
-          (insert "------------------------------------------------------------\n")
-          (insert (format "Command: %s" f))
-          (insert (format "\tKey Sequence:%s\n\n"
-                          (or (mapconcat
-                               'key-description
-                               (where-is-internal f) " ")
-                              "Not bound to any key")))
-          (insert (documentation f))
-          (insert "\n\n")))
+          (let ((key (where-is-internal f)))
+            (insert "------------------------------------------------------------\n")
+            (insert (format "** %s" f))
+            (if key 
+                (insert (format "\tKey Sequence:%s\n\n"
+                                (mapconcat
+                                 'key-description
+                                 key " ")))
+              (insert " No global keybinding\n\n"))
+            (insert
+             (or (documentation f)
+                 ""))
+            (insert "\n\n"))))
        (emacspeak-list-emacspeak-commands))
       (save-buffer)))
   (emacspeak-auditory-icon 'task-done))
 
+
+
 ;;}}}
 ;;{{{  translate faces to voices
 (defun voice-lock-voiceify-faces ()
@@ -2241,6 +2341,11 @@ Warning! Contents of file filename will be overwritten."
          (put-text-property (match-beginning 0)
                             (match-end 0)
                             'personality 'inaudible)))))))
+(defvar emacspeak-completions-current-prefix nil
+  "Prefix typed in the minibuffer before completions was
+invoked")
+
+(make-variable-buffer-local 'emacspeak-completions-current-prefix)
 
 (defun emacspeak-switch-to-completions-window ()
   "Jump to the *Completions* buffer if it is active.  We make
@@ -2248,7 +2353,8 @@ the current minibuffer contents (which is obviously the
 prefix for each entry in the completions buffer) inaudible
 to reduce chatter."
   (interactive)
-  (declare (special voice-lock-mode))
+  (declare (special voice-lock-mode
+                    emacspeak-completions-current-prefix))
   (let ((completions-buffer (get-buffer "*Completions*"))
         (current-entry (emacspeak-get-minibuffer-contents)))
     (cond
@@ -2262,6 +2368,7 @@ to reduce chatter."
         (setq voice-lock-mode t)
         (when (and  current-entry 
                     (> (length current-entry) 0))
+          (setq emacspeak-completions-current-prefix current-entry)
           (emacspeak-make-string-inaudible current-entry))
         (dtk-toggle-splitting-on-white-space)
         (dtk-speak
@@ -2269,9 +2376,291 @@ to reduce chatter."
       (emacspeak-auditory-icon 'select-object))
      (t (message "No completions")))))
 
+(defun emacspeak-completions-move-to-completion-group()
+  "Move to group of choices beginning with character last
+typed. If no such group exists, then we dont move. "
+  (interactive)
+  (declare (special last-input-char
+                    emacspeak-completions-current-prefix))
+  (let ((pattern (format "[ \t\n]%s%c"
+                         (or
+                          emacspeak-completions-current-prefix "")
+                         last-input-char))
+        (case-fold-search t))
+    (when (re-search-forward pattern nil t)
+      (emacspeak-auditory-icon 'search-hit))
+    (dtk-speak
+     (emacspeak-get-current-completion-from-completions ))))
+(declaim (special completion-list-mode-map))
+(let ((chars
+       "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"))
+  (loop for char across chars
+        do
+        (define-key completion-list-mode-map
+          (format "%c" char)
+          'emacspeak-completions-move-to-completion-group)))
+
 
 ;;}}}
 
+;;}}}
+;;{{{ emacspeak clipboard
+(eval-when (load)
+(condition-case nil
+    (unless (file-exists-p emacspeak-resource-directory)
+      (make-directory emacspeak-resource-directory))
+  (error (message "Make sure you have an Emacspeak resource directory %s"
+            emacspeak-resource-directory))))
+
+(defvar emacspeak-clipboard-file
+   (concat emacspeak-resource-directory "/" "clipboard")
+  "File used to save Emacspeak clipboard.
+The emacspeak clipboard provides a convenient mechnaism for exchanging
+information between different emacs sessions.")
+
+(defun emacspeak-clipboard-copy (start end &optional prompt)
+  "Copy contents of the region to the emacspeak clipboard.  Previous
+contents of the clipboard will be overwritten.  The Emacspeak
+clipboard is a convenient way of sharing information between
+independent Emacspeak sessions running on the same or different
+machines.  Do not use this for sharing information within an Emacs
+session --Emacs' register commands are far more efficient and
+light-weight.  Optional interactive prefix arg results in Emacspeak
+prompting for the clipboard file to use."
+  (interactive "r\nP")
+  (declare (special emacspeak-resource-directory emacspeak-clipboard-file))
+  (let ((clip (buffer-substring-no-properties start end ))
+        (clipboard-file
+         (if prompt
+             (read-file-name "Copy region to clipboard file: "
+                             emacspeak-resource-directory
+                             emacspeak-clipboard-file)
+           emacspeak-clipboard-file))
+        (clipboard nil))
+    (setq clipboard (find-file-noselect  clipboard-file))
+    (let ((emacspeak-speak-messages nil))
+      (save-excursion
+        (set-buffer clipboard)
+        (erase-buffer)
+        (insert clip)
+        (save-buffer)))
+    (message "Copied %s lines to Emacspeak clipboard %s"
+             (count-lines start end)
+             clipboard-file)))
+
+
+
+(defun emacspeak-clipboard-paste (&optional prompt)
+  "Yank contents of the Emacspeak clipboard at point.  The Emacspeak
+clipboard is a convenient way of sharing information between
+independent Emacspeak sessions running on the same or different
+machines.  Do not use this for sharing information within an Emacs
+session --Emacs' register commands are far more efficient and
+light-weight.  Optional interactive prefix arg results in Emacspeak
+prompting for the clipboard file to use."
+  (interactive "P")
+  (declare (special emacspeak-resource-directory emacspeak-clipboard-file))
+  (let ((clip nil)
+        (start (point))
+        (clipboard-file
+         (if prompt
+             (read-file-name "Yank from  clipboard file: "
+                             emacspeak-resource-directory
+                             emacspeak-clipboard-file)
+           emacspeak-clipboard-file))
+        (clipboard nil))
+    (setq clipboard (find-file-noselect  clipboard-file))
+    (setq clip 
+          (let ((emacspeak-speak-messages nil))
+            (save-excursion
+              (set-buffer clipboard)
+              (buffer-string))))
+    (insert clip)
+    (message "Yanked %s lines from  Emacspeak clipboard %s"
+             (count-lines start (point))
+             clipboard-file)))
+
+;;}}}
+;;{{{ utilities 
+
+(defun emacspeak-speak-show-list-variable (var)
+  "Convenience command to view Emacs variables that are long lists.
+Prompts for a variable name and displays its value in a separate buffer.
+Lists are displayed one element per line."
+  (interactive 
+   (list 
+    (read-minibuffer "Display variable: ")))
+  (let ((buffer (get-buffer-create
+                 (format "*emacspeak-%s*"
+                         var)))
+        (symbol (symbol-value var)))
+    (save-excursion
+      (set-buffer buffer)
+      (erase-buffer)
+      (cond
+       ((listp symbol)
+        (loop for element in symbol
+              do
+              (insert (format "%s\n"
+                              element))))
+       (t (insert (format "%s\n" symbol))))
+      (goto-char (point-min)))
+    (pop-to-buffer buffer)
+    (emacspeak-auditory-icon 'open-object)
+    (emacspeak-speak-mode-line)))
+
+;;}}}
+;;{{{ mark convenience commands
+(defsubst emacspeak-mark-speak-mark-line()
+  (emacspeak-auditory-icon 'mark-object )
+  (ems-set-personality-temporarily (point) (1+ (point))
+                                   'paul-animated
+                                   (emacspeak-speak-line)))
+
+(defun emacspeak-mark-forward-mark ()
+  "Cycle forward through the mark ring"
+  (interactive)
+  (set-mark-command t)
+  (when (interactive-p )
+    (emacspeak-mark-speak-mark-line)))
+
+(defun emacspeak-mark-backward-mark ()
+  "Cycle backward through the mark ring"
+  (interactive)
+  (declare (special mark-ring))
+  (let ((target  (car (last mark-ring ))))
+    (cond
+ (target
+      (setq mark-ring
+            (cons (copy-marker (mark-marker)) 
+                  (butlast mark-ring 1)))
+      (set-marker (mark-marker) (+ 0 target)
+                  (current-buffer))
+      (move-marker target nil)
+      (goto-char (mark t))
+      (when (interactive-p)
+        (emacspeak-mark-speak-mark-line)))
+(t (message "No previous mark to move to")))))
+
+
+;;}}}
+;;{{{ labelled frames
+
+(defun emacspeak-frame-label-or-switch-to-labelled-frame (label
+                                                          &optional prefix)
+  "Label current frame. With optional prefix argument,
+switch to a labelled frame."
+  (interactive "sFrame label:\nP")
+  (cond
+   (prefix
+    (let ((frame (loop for f in (frame-list)
+                       if (string= label (frame-parameter f 'emacspeak-label))
+                       return f)))
+      (cond
+       ( frame
+         (select-frame frame)
+         (emacspeak-auditory-icon 'select-object)
+         nil
+
+         (emacspeak-speak-mode-line))
+       (t
+        (message "No frame labelled %s" label)))))
+   (t (modify-frame-parameters (selected-frame)
+                               (list (cons 'emacspeak-label label))))))
+
+(defun emacspeak-next-frame ()
+  "Move to next frame"
+  (interactive)
+  (other-frame 1)
+  (emacspeak-auditory-icon 'select-object)
+  (emacspeak-speak-mode-line))
+
+(defun emacspeak-previous-frame ()
+  "Move to next frame"
+  (interactive)
+  (other-frame -1)
+  (emacspeak-auditory-icon 'select-object)
+  (emacspeak-speak-mode-line))
+
+;;}}}
+;;{{{  readng different displays of same buffer
+
+(defun emacspeak-speak-this-buffer-other-window-display (&optional arg)
+  "Emacs allows you display the same buffer in multiple
+windows or frames.
+These different windows can 
+display different portions of the buffer.
+This is equivalent to leaving a book open at places at once.
+This command allows you to listen to the places where you
+have left the book open. The number used to invoke this
+command
+specifies which of the displays you wish to speak. Typically
+you will have two or at most three such displays open.
+The current display is 0, the next is 1, and so on."
+  (interactive "P")
+  (let ((window (or arg 
+         (condition-case nil
+             (read (format "%c" last-input-event ))
+           (error nil ))))
+        (win nil)
+        (window-list (get-buffer-window-list
+                      (current-buffer)
+                      nil 'visible)))
+        (or (numberp window)
+            (setq window
+                  (read-minibuffer "Window    to speak")))
+        (setq win
+              (nth (% window (length window-list ))
+                   window-list))
+        (emacspeak-speak-region
+         (window-start win)
+         (window-end win))))
+
+(defun emacspeak-speak-this-buffer-previous-display ()
+  "Speak this buffer as displayed in a `previous' window.
+See documentation for command
+emacspeak-speak-this-buffer-other-window-display for the
+meaning of `previous'."
+  (interactive)
+(let ((count (length (get-buffer-window-list
+                      (current-buffer)
+                      nil 'visible))))
+  (emacspeak-speak-this-buffer-other-window-display (1-  count))))
+
+(defun emacspeak-speak-this-buffer-next-display ()
+  "Speak this buffer as displayed in a `previous' window.
+See documentation for command
+emacspeak-speak-this-buffer-other-window-display for the
+meaning of `previous'."
+  (interactive)
+  (emacspeak-speak-this-buffer-other-window-display  1))
+
+;;}}}
+;;{{{  Emacspeak News and Documentation
+
+(defun emacspeak-view-emacspeak-news ()
+  "Display info on recent changes to Emacspeak."
+  (interactive)
+  (declare (special emacspeak-dir
+                    emacspeak-version))
+  (find-file-read-only (expand-file-name "NEWS"
+                                         emacspeak-dir))
+  (emacspeak-auditory-icon 'news)
+  (dtk-speak
+   (format "Welcome to Emacspeak %s News And Features"
+           emacspeak-version)))
+
+(defun emacspeak-view-emacspeak-doc ()
+  "Display automatically generated documentation on
+Emacspeak commands"
+  (interactive)
+  (declare (special emacspeak-dir))
+  (find-file-read-only (expand-file-name "DOC"
+                                         emacspeak-dir))
+  (outline-minor-mode 1)
+  (emacspeak-auditory-icon 'open-object)
+  (emacspeak-speak-mode-line))
+
 ;;}}}
 (provide 'emacspeak-speak )
 ;;{{{ end of file 
