@@ -1,4 +1,5 @@
-;;; $Id: emacspeak-table-ui.el,v 7.0 1997/11/13 15:32:30 raman Exp $
+;;; emacspeak-table-ui.el --- Emacspeak's current notion of an ideal table UI
+;;; $Id: emacspeak-table-ui.el,v 8.0 1998/05/01 14:15:46 raman Exp $
 ;;; $Author: raman $ 
 ;;; Description: Emacspeak table handling module
 ;;; Keywords:emacspeak, audio interface to emacs tables are structured
@@ -7,14 +8,14 @@
 ;;; LCD Archive Entry:
 ;;; emacspeak| T. V. Raman |raman@adobe.com
 ;;; A speech interface to Emacs |
-;;; $Date: 1997/11/13 15:32:30 $ |
-;;;  $Revision: 7.0 $ | 
+;;; $Date: 1998/05/01 14:15:46 $ |
+;;;  $Revision: 8.0 $ | 
 ;;; Location undetermined
 ;;;
 
 ;;}}}
 ;;{{{  Copyright:
-;;;Copyright (C) 1995, 1996, 1997  T. V. Raman  Adobe Systems Incorporated
+;;;Copyright (C) 1995, 1996, 1997, 1998   T. V. Raman  Adobe Systems Incorporated
 ;;; Copyright (c) 1995 by T. V. Raman Adobe Systems Incorporated 
 ;;; All Rights Reserved. 
 ;;;
@@ -178,14 +179,20 @@ major-mode 'emacspeak-table-mode) (setq mode-name "table") (put-text-property
   "Bring visual display in sync with internal representation"
   (declare (special emacspeak-table positions))
   (let ((row (emacspeak-table-current-row emacspeak-table))
-        (column (emacspeak-table-current-column emacspeak-table)))
-        (goto-char
-         (or 
-         (gethash
-          (intern
-           (format "element:%s:%s" row column))
-          positions)
-         (point)))))
+        (column (emacspeak-table-current-column
+                 emacspeak-table))
+        (col (current-column))
+        (width (frame-width)))
+    (goto-char
+     (or 
+      (gethash
+       (intern
+        (format "element:%s:%s" row column))
+       positions)
+      (point)))
+    (scroll-left (- (current-column)
+                    (+ (/ width  2)
+(window-hscroll))))))
 
 
 (defsubst  emacspeak-table-speak-coordinates ()
@@ -409,38 +416,40 @@ the documentation on the table browser."
     (kill-buffer data )
     (save-excursion
       (set-buffer buffer)
+      (setq truncate-lines t)
       (let ((inhibit-read-only t))
-      (erase-buffer)
-      (set (make-local-variable 'emacspeak-table) table)
-      (set (make-local-variable 'positions) (make-hash-table))
-      (setq buffer-file-name filename)
-      (setq count (1-  (emacspeak-table-num-columns table)))
-      (loop for row across (emacspeak-table-elements table)
-            do
-            (loop for element across row
-                  do 
-                  (setf
-                   (gethash
-                    (intern (format "element:%s:%s" i j ))
-                    positions)
-                   (point))
-                  (insert
-                   (format "%s%s"
-                           (emacspeak-table-this-element table i j )
-                           (if (=  j count)
-                               "\n"
-                             "\t")))
-                  (put-text-property column-start (point)
-                                     'column j)
-                  (setq column-start (point))
-                  (incf j))
-            (setq j 0)
-            (put-text-property row-start (point) 'row i)
-            (setq row-start (point))
-            (incf i))
-      (emacspeak-table-mode)
-      (goto-char (point-min))))
-    (switch-to-buffer buffer)))
+        (erase-buffer)
+        (set (make-local-variable 'emacspeak-table) table)
+        (set (make-local-variable 'positions) (make-hash-table))
+        (setq buffer-file-name filename)
+        (setq count (1-  (emacspeak-table-num-columns table)))
+        (loop for row across (emacspeak-table-elements table)
+              do
+              (loop for element across row
+                    do 
+                    (setf
+                     (gethash
+                      (intern (format "element:%s:%s" i j ))
+                      positions)
+                     (point))
+                    (insert
+                     (format "%s%s"
+                             (emacspeak-table-this-element table i j )
+                             (if (=  j count)
+                                 "\n"
+                               "\t")))
+                    (put-text-property column-start (point)
+                                       'column j)
+                    (setq column-start (point))
+                    (incf j))
+              (setq j 0)
+              (put-text-property row-start (point) 'row i)
+              (setq row-start (point))
+              (incf i))
+        (emacspeak-table-mode)
+        (goto-char (point-min))))
+    (switch-to-buffer buffer)
+    (setq truncate-lines t)))
 
 (emacspeak-fix-interactive-command-if-necessary 'emacspeak-table-find-file)
 
