@@ -1,4 +1,5 @@
-;;; $Id: emacspeak-view-process.el,v 7.0 1997/11/13 15:32:30 raman Exp $
+;;; emacspeak-view-process.el --- Speech enable View Processes -- A powerful task manager
+;;; $Id: emacspeak-view-process.el,v 8.0 1998/05/01 14:15:46 raman Exp $
 ;;; $Author: raman $ 
 ;;; Description: Emacspeak extension for flexible viewing of processes
 ;;; Keywords:emacspeak, audio interface to emacs administering processes
@@ -7,14 +8,14 @@
 ;;; LCD Archive Entry:
 ;;; emacspeak| T. V. Raman |raman@adobe.com
 ;;; A speech interface to Emacs |
-;;; $Date: 1997/11/13 15:32:30 $ |
-;;;  $Revision: 7.0 $ | 
+;;; $Date: 1998/05/01 14:15:46 $ |
+;;;  $Revision: 8.0 $ | 
 ;;; Location undetermined
 ;;;
 
 ;;}}}
 ;;{{{  Copyright:
-;;;Copyright (C) 1995, 1996, 1997  T. V. Raman  Adobe Systems Incorporated
+;;;Copyright (C) 1995, 1996, 1997, 1998   T. V. Raman  Adobe Systems Incorporated
 ;;; Copyright (c) 1995 by T. V. Raman Adobe Systems Incorporated 
 ;;; All Rights Reserved. 
 ;;;
@@ -57,6 +58,7 @@
 
 ;;}}}
 ;;{{{ Advice interactive commands:
+
 (defadvice View-process-mode (after emacspeak pre act comp)
   "Provide auditory feedback"
   (when (interactive-p)
@@ -68,7 +70,7 @@
                                                           comp)
   "Provide auditory feedback"
   (when (interactive-p)
-    (emacspeak-auditory-icon 'scroll)
+    (emacspeak-auditory-icon 'select-object)
     (View-process-show-pid-and-command)))
 
 (defadvice  View-process-next-field (after emacspeak pre act comp)
@@ -136,6 +138,43 @@
   (when  (interactive-p)
     (emacspeak-auditory-icon 'off)
     (message "Deleted itimer")))
+(defadvice View-process-mark-childs-in-current-line (around emacspeak pre act comp)
+  "Display number of processes marked"
+  (cond
+   ((interactive-p)
+    (let ((count (length View-process-pid-mark-alist)))
+      ad-do-it
+      (emacspeak-auditory-icon 'mark-object)
+      (message "Marked %s child processes"
+               (- (length View-process-pid-mark-alist)
+                  (1+ count)))))
+   (t ad-do-it))
+  ad-return-value)
+
+(defadvice View-process-unmark-all (around emacspeak pre act comp)
+  "Display number of processes were unmarked"
+  (cond
+   ((interactive-p)
+    (let ((count (length View-process-pid-mark-alist)))
+      ad-do-it
+      (emacspeak-auditory-icon 'deselect-object)
+      (message "Unmarked %s  processes"
+                count)))
+   (t ad-do-it))
+  ad-return-value)
+
+(defadvice View-process-unmark-current-line(after emacspeak
+                                                  pre act comp)
+  "Provide auditory icon"
+  (when (interactive-p)
+    (emacspeak-auditory-icon 'deselect-object)
+(View-process-show-pid-and-command)))
+(defadvice View-process-mark-current-line(after emacspeak
+                                                  pre act comp)
+  "Provide auditory icon"
+  (when (interactive-p)
+    (emacspeak-auditory-icon 'mark-object)
+(View-process-show-pid-and-command)))
 
 ;;}}}
 ;;{{{ voice locking
