@@ -1,4 +1,5 @@
-;;; $Id: emacspeak-view.el,v 7.0 1997/11/13 15:32:30 raman Exp $
+;;; emacspeak-view.el --- Speech enable View mode -- Efficient browsing of read-only content
+;;; $Id: emacspeak-view.el,v 8.0 1998/05/01 14:15:46 raman Exp $
 ;;; $Author: raman $ 
 ;;; DescriptionEmacspeak extensions for view
 ;;; Keywords:emacspeak, audio interface to emacs, view-mode
@@ -7,14 +8,14 @@
 ;;; LCD Archive Entry:
 ;;; emacspeak| T. V. Raman |raman@adobe.com
 ;;; A speech interface to Emacs |
-;;; $Date: 1997/11/13 15:32:30 $ |
-;;;  $Revision: 7.0 $ | 
+;;; $Date: 1998/05/01 14:15:46 $ |
+;;;  $Revision: 8.0 $ | 
 ;;; Location undetermined
 ;;;
 
 ;;}}}
 ;;{{{  Copyright:
-;;;Copyright (C) 1995, 1996, 1997  T. V. Raman  Adobe Systems Incorporated
+;;;Copyright (C) 1995, 1996, 1997, 1998   T. V. Raman  Adobe Systems Incorporated
 ;;; Copyright (c) 1996 by T. V. Raman 
 ;;; All Rights Reserved. 
 ;;;
@@ -53,7 +54,8 @@
 (declaim (special emacspeak-prefix))
 (add-hook 'view-mode-hook
           (function (lambda ()
-                      (local-unset-key emacspeak-prefix ))))
+                      (local-unset-key emacspeak-prefix )
+(emacspeak-view-setup-keys))))
 ;;; Generate automatic advise:
 
 (mapcar 'emacspeak-fix-interactive-command-if-necessary
@@ -64,13 +66,38 @@ View-search-regexp-backward
 view-buffer-other-window
 View-search-regexp-forward))
 
+;;}}}
+;;{{{ additional interactive commands
+
+(defun emacspeak-view-line-to-top ()
+  "Moves current line to top of window"
+  (interactive)
+  (recenter 0)
+  (emacspeak-speak-line)
+  (emacspeak-auditory-icon 'select-object))
+
 ;;}}}
 ;;{{{ Advise additional interactive commands:
+(defadvice view-mode (after emacspeak pre act comp)
+  "Announce what happened"
+  (when (interactive-p)
+    (emacspeak-auditory-icon 'open-object)
+    (message "Entered view mode Press %s to exit"
+(key-description
+             (where-is-internal 'View-exit view-mode-map 'firstonly)))))
+
 (defadvice View-quit (after emacspeak pre act comp)
   "Provide auditory feedback."
   (when (interactive-p)
     (emacspeak-auditory-icon 'close-object)
     (emacspeak-speak-mode-line)))
+
+(defadvice View-exit (after emacspeak pre act comp)
+  "Provide auditory feedback."
+  (when (interactive-p)
+    (emacspeak-auditory-icon 'close-object)
+    (emacspeak-speak-mode-line)))
+
 (defadvice View-leave (after emacspeak pre act comp)
   "Provide auditory feedback."
   (when (interactive-p)
@@ -257,6 +284,64 @@ View-search-regexp-forward))
     (emacspeak-auditory-icon 'scroll)
     (dtk-speak (emacspeak-get-window-contents))))
 
+;;}}}
+;;{{{ bind convenience keys
+(defvar emacspeak-view-keys-optimized nil
+  "Records if we have already optimized Emacspeak
+keybindings for view mode")
+
+(defvar emacspeak-view-edit-commands
+  (list 'emacspeak-self-insert-command
+        'completion-separator-self-insert-autofilling
+        'completion-separator-self-insert-command
+        'delete-char
+        'completion-kill-region
+        )
+  "Editting commands that emacspeak should rebind in view mode")
+
+(defun emacspeak-view-optimize-view-keys()
+  "optimize keybindings for emacspeak in view mode"
+  (declare (special emacspeak-view-edit-commands
+                    emacspeak-view-keys-optimized
+                    view-mode-map emacspeak-keymap))
+  (unless emacspeak-view-keys-optimized
+    (setq emacspeak-view-keys-optimized t)
+    (loop for edit-command in emacspeak-view-edit-commands
+          do
+          (let ((edit-keys (where-is-internal edit-command view-mode-map)))
+            (loop for key in edit-keys 
+                  do
+                  (let ((command (lookup-key emacspeak-keymap key)))
+                    (when command
+                      (define-key view-mode-map key command))))))))
+
+(defun emacspeak-view-setup-keys()
+  "Setup emacspeak convenience keys"
+  (declare (special view-mode-map))
+  (loop for i from 0 to 9
+        do
+        (define-key view-mode-map
+          (format "%s" i)
+          'emacspeak-speak-predefined-window))
+;;;convenience keys
+  (define-key view-mode-map "\C-j" 'emacspeak-hide-speak-block-sans-prefix)
+  (define-key view-mode-map "\M-n"
+    'outline-next-visible-heading)
+  (define-key view-mode-map "\M-p" 'outline-previous-visible-heading)
+  (define-key view-mode-map " " 'scroll-up)
+  (define-key view-mode-map '[DEL] 'scroll-down)
+  (define-key view-mode-map "P" 'dtk-pause)
+  (define-key view-mode-map "R" 'dtk-resume)
+  (define-key view-mode-map "S" 'dtk-stop)
+  (define-key view-mode-map "t" 'emacspeak-view-line-to-top)
+  (define-key view-mode-map ","
+    'emacspeak-speak-current-window)
+  (define-key view-mode-map "\M-d"
+    'emacspeak-pronounce-dispatch)
+  (define-key view-mode-map "c" 'emacspeak-speak-char)
+  (define-key view-mode-map "w" 'emacspeak-speak-word)
+  (emacspeak-view-optimize-view-keys))
+
 ;;}}}
 (provide  'emacspeak-view)
 ;;{{{  emacs local variables 
