@@ -1,20 +1,21 @@
-;;; $Id: emacspeak-w3.el,v 7.0 1997/11/13 15:32:30 raman Exp $
+;;; emacspeak-w3.el --- Speech enable W3 WWW browser -- includes ACSS Support
+;;; $Id: emacspeak-w3.el,v 8.0 1998/05/01 14:15:46 raman Exp $
 ;;; $Author: raman $ 
 ;;; Description:  Emacspeak enhancements for W3
-;;; Keywords: Emacspeak, W3, WWW, Spoken Output, Dectalk
+;;; Keywords: Emacspeak, W3, WWW
 ;;{{{  LCD Archive entry: 
 
 ;;; LCD Archive Entry:
 ;;; emacspeak| T. V. Raman |raman@adobe.com 
 ;;; A speech interface to Emacs |
-;;; $Date: 1997/11/13 15:32:30 $ |
-;;;  $Revision: 7.0 $ | 
+;;; $Date: 1998/05/01 14:15:46 $ |
+;;;  $Revision: 8.0 $ | 
 ;;; Location undetermined
 ;;;
 
 ;;}}}
 ;;{{{  Copyright:
-;;;Copyright (C) 1995, 1996, 1997  T. V. Raman  Adobe Systems Incorporated
+;;;Copyright (C) 1995, 1996, 1997, 1998   T. V. Raman  Adobe Systems Incorporated
 ;;; Copyright (c) 1994, 1995 by Digital Equipment Corporation.
 ;;; All Rights Reserved. 
 ;;;
@@ -42,7 +43,15 @@
 (require 'emacspeak-sounds)
 ;;{{{  Introduction:
 
-;;; Ensure that speech support for W3 gets installed and loaded correctly.
+;;; Commentary:
+
+;;; Ensure that speech support for W3 gets installed and
+;;; loaded correctly.
+;;; The emacs W3 browser comes with builtin support for
+;;; Emacspeak and ACSS
+
+;;; Code:
+
 
 ;;}}}
 ;;{{{  additional advice
@@ -56,17 +65,21 @@
 ;;{{{ setup
 
 (declaim (special w3-echo-link
- url-show-status
- w3-mode-map))
+                  url-show-status
+                  w3-mode-map))
 
-              (condition-case nil
-                  (progn (require 'w3-speak)
-                         (add-hook 'w3-mode-hook 'w3-speak-mode-hook)
-                         (setq w3-echo-link 'text)
-                         (setq url-show-status nil))
-                (error (emacspeak-auditory-icon 'warn-user)
-                       (message
-                        "You appear to be using an old version of W3
+(condition-case nil
+    (progn (require 'w3-speak)
+           (add-hook 'w3-mode-hook 'w3-speak-mode-hook)
+           (setq w3-echo-link 'text)
+           (if (locate-library "w3-speak-table")
+             (load-library "w3-speak-table")
+             (message
+              "Please upgrade to W3 4.0.18 for spoken table support"))
+           (setq url-show-status nil))
+  (error (emacspeak-auditory-icon 'warn-user)
+         (message
+          "You appear to be using an old version of W3
 that is no longer supported by Emacspeak.")))
 
 (add-hook 'w3-mode-hook
@@ -108,6 +121,40 @@ w3-read-html-bookmarks)
       do
       (emacspeak-fix-interactive-command-if-necessary f ))
 
+;;}}}
+;;{{{ fixup images
+(declaim (special w3-version))
+;;; smart image handling only works in w3 4.0
+(when
+    (string-match  "4\\.0" w3-version)
+; simple heuristic to detect silly bullets and dots
+; (by Greg Stark <gsstark@mit.edu>, enriched with regexp)
+
+(defvar w3-min-img-size 16
+  "*Image size under which the alt string is replaced by `w3-dummy-img-alt-repl'.
+15 is a bit aggressive, 5 pixels would be safer")
+
+(defvar w3-dummy-img-re
+  "\\(bullet\\|\\b\\(boule\\|dot\\|pebble[0-9]*[a-z]?[0-9]*\\|pixel\\)\\b\\)"
+  "Image name regexp for which the alt string is replaced by `w3-dummy-img-alt-repl'.")
+
+(defvar w3-dummy-img-alt-repl "@"
+  "*Dummy img alt replacement")
+
+(declaim (special  w3-auto-image-alt))
+(setq w3-auto-image-alt
+      (function
+       (lambda (s)
+         (declare (special width height ))
+         (if (or (and (stringp height)
+                      (< (string-to-int height) w3-min-img-size))
+                 (and (stringp width)
+                      (< (string-to-int width) w3-min-img-size))
+                 (string-match w3-dummy-img-re s))
+             w3-dummy-img-alt-repl
+           (concat "[" (file-name-sans-extension s)
+                   "]"))))))
+
 ;;}}}
 (provide 'emacspeak-w3)
 ;;{{{  emacs local variables 
