@@ -1,4 +1,5 @@
-;;; $Id: emacspeak.el,v 7.0 1997/11/13 15:32:30 raman Exp $
+;;; emacspeak.el --- Emacspeak -- The Complete Audio Desktop
+;;; $Id: emacspeak.el,v 8.0 1998/05/01 14:15:46 raman Exp $
 ;;; $Author: raman $ 
 ;;; Description:  Emacspeak: A speech interface to Emacs
 ;;; Keywords: Emacspeak, Speech, Dectalk,
@@ -7,14 +8,14 @@
 ;;; LCD Archive Entry:
 ;;; emacspeak| T. V. Raman |raman@adobe.com 
 ;;; A speech interface to Emacs |
-;;; $Date: 1997/11/13 15:32:30 $ |
-;;;  $Revision: 7.0 $ | 
+;;; $Date: 1998/05/01 14:15:46 $ |
+;;;  $Revision: 8.0 $ | 
 ;;; Location undetermined
 ;;;
 
 ;;}}}
 ;;{{{  Copyright:
-;;;Copyright (C) 1995, 1996, 1997  T. V. Raman  Adobe Systems Incorporated
+;;;Copyright (C) 1995, 1996, 1997, 1998   T. V. Raman  Adobe Systems Incorporated
 ;;; Copyright (c) 1994, 1995 by Digital Equipment Corporation.
 ;;; All Rights Reserved. 
 ;;;
@@ -54,7 +55,7 @@
 ;;{{{  Setting up things: 
 
 (defconst emacspeak-version
-  (let ((x "$Revision: 7.0 $"))
+  (let ((x "$Revision: 8.0 $"))
     (string-match "[0-9.]+" x)
     (substring x (match-beginning 0)
                (match-end 0)))
@@ -65,17 +66,6 @@
 
 ;;}}}
 ;;{{{ add autoloads and set some load hooks
-(autoload 'emacspeak-remote-startup-local-server
-   "emacspeak-remote"
-"Startup  speech server.
-Use this when you are running emacspeak on a remote machine and want
-to have speech output on the local desktop.
-Unlike the simpler rsh based remote-tcl solution
-(see file remote-tcl in the emacspeak distribution)
-this facility allows you
-to get remote speech feedback in cases where the remote machine cannot
-use rsh on your local desktop."
-  t)
 
 (autoload 'emacspeak-remote-connect-to-server "emacspeak-remote"
 "Connect to a remote speech server.
@@ -87,6 +77,17 @@ this command emacspeak-remote-connect-to-server allows you
 to get remote speech feedback in cases where the remote machine cannot
 use rsh on your local desktop."
   t)
+(autoload 'emacspeak-remote-quick-connect-to-server "emacspeak-remote"
+"Connect to a remote speech server.
+Use this when you are running emacspeak on a remote machine and want
+to have speech output on the local desktop.
+Unlike the simpler rsh based remote-tcl solution
+(see file remote-tcl in the emacspeak distribution)
+this command emacspeak-remote-connect-to-server allows you
+to get remote speech feedback in cases where the remote machine cannot
+use rsh on your local desktop."
+  t)
+
 
 (autoload 'emacspeak-eterm-remote-term "emacspeak-eterm"
    "Create a terminal to rlogin into a remote host" t)
@@ -106,7 +107,6 @@ use rsh on your local desktop."
 
 (defvar emacspeak-emacs-commands-to-fix 
   '(
-    ad-compile-function
     add-completions-from-file
     append-to-file
     append-to-register
@@ -132,6 +132,9 @@ use rsh on your local desktop."
     dtk-set-character-scale
     dtk-set-rate
     emacspeak-dial-dtk
+    emacspeak-keymap-choose-new-emacspeak-prefix
+    emacspeak-frame-label-or-switch-to-labelled-frame
+    emacspeak-generate-documentation
     find-file
     find-file-literally
     find-file-literally
@@ -254,20 +257,20 @@ See the online documentation for individual commands and functions for
  preliminary. The source code documentation is up-to-date, please use it.  "
   
   (interactive)
-  (declare (special dtk-program dtk-drivers-alist dtk-driver-prefix
+  (declare (special dtk-program dtk-servers-alist dtk-server-prefix
                     emacspeak-emacs-commands-to-fix))
-  (let ((driver-flag nil ))
+  (let ((server-flag nil ))
     (load-library "dtk-speak")
     (unless  dtk-program
       (ding t)
       (sit-for 1)
       (ding t)
-      (setq driver-flag t 
-            dtk-program (completing-read "Select Dectalk driver:"
-                                         dtk-drivers-alist  nil
-                                         t dtk-driver-prefix )))
+      (setq server-flag t 
+            dtk-program (completing-read "Select Dectalk server:"
+                                         dtk-servers-alist  nil
+                                         t dtk-server-prefix )))
     (dtk-initialize)
-    (when driver-flag
+    (when server-flag
       (dtk-speak
        "Make sure you  set the environment variable DTK_PROGRAM next
 time!"))
@@ -277,6 +280,7 @@ time!"))
     (load-library "emacspeak-keymap")
     (load-library "emacspeak-advice")
     (load-library "emacspeak-replace")
+    (require 'emacspeak-buff-menu)
     (mapcar 'emacspeak-fix-interactive-command-if-necessary 
             emacspeak-emacs-commands-to-fix)
     (run-hooks 'emacspeak-startup-hook)
@@ -299,6 +303,7 @@ circuits are functioning perfectly! "
 
 ;;}}}
 ;;{{{ autoloads
+
 (autoload 'emacspeak-table-find-file "emacspeak-table-ui"
   "Browse tables. Beginners: please do C-e C-t and specify one of the
 .tab files in the tables subdirectory of emacspeak.
@@ -326,7 +331,16 @@ Useful in reading email, block comments in program source etc." t)
   "Speak a block of text sans the prefix that appears on every line of
 the block. Use in conjunction with emacspeak-hide-or-expose-block"
    t)
+;;; Realaudio
+(autoload 'emacspeak-realaudio "emacspeak-realaudio"
+  "Single click interface to RealAudio" t)
+(autoload 'emacspeak-realaudio-play-url-at-point "emacspeak-realaudio"
+  "Single click interface to RealAudio" t)
 
+(autoload 'emacspeak-realaudio-play "emacspeak-realaudio"
+  "Single click interface to RealAudio" t)
+(autoload 'emacspeak-realaudio-stop "emacspeak-realaudio"
+  "Single click interface to RealAudio" t)
 
 ;;}}}
 ;;{{{  Advice for Emacs subsystems:
@@ -351,6 +365,24 @@ the block. Use in conjunction with emacspeak-hide-or-expose-block"
           '("imenu"
                         (load "emacspeak-imenu")))
 
+;;}}}
+;;{{{  jde
+(add-hook 'after-load-alist
+          '("jde"
+                        (load "emacspeak-jde")))
+
+;;}}}
+;;{{{  speedbar
+(add-hook 'after-load-alist
+          '("speedbar"
+                        (load "emacspeak-speedbar")))
+
+;;}}}
+;;{{{  python
+(add-hook 'after-load-alist
+          '("python-mode"
+                        (load "emacspeak-python")))
+
 ;;}}}
 ;;{{{  tempo
 (add-hook 'after-load-alist
@@ -383,16 +415,7 @@ the block. Use in conjunction with emacspeak-hide-or-expose-block"
                         (load "emacspeak-bookmark")))
 
 ;;}}}
-;;{{{  buff-menu
 
-(add-hook 'after-load-alist
-          '("buff-menu"
-                        (load "emacspeak-buff-menu")))
-(defadvice list-buffers (before first pre act comp)
-  "Make sure emacspeak advice is loaded"
-  (require 'emacspeak-buff-menu))
-
-;;}}}
 ;;{{{  tar
 
 (add-hook 'after-load-alist
@@ -505,6 +528,24 @@ the block. Use in conjunction with emacspeak-hide-or-expose-block"
           '("wid-edit"
                         (load "emacspeak-widget")))
 ;;}}}
+;;{{{ custom
+
+(add-hook 'after-load-alist
+          '("custom"
+                        (load "emacspeak-custom")))
+;;}}}
+;;{{{ mspools
+
+(add-hook 'after-load-alist
+          '("mspools"
+                        (load "emacspeak-mspools")))
+;;}}}
+;;{{{ winring
+
+(add-hook 'after-load-alist
+          '("winring"
+                        (load "emacspeak-winring")))
+;;}}}
 ;;{{{ w3
 
 (add-hook 'after-load-alist
@@ -691,10 +732,10 @@ the block. Use in conjunction with emacspeak-hide-or-expose-block"
 (if (locate-library "ediff-init")
 (add-hook 'ediff-load-hook
           (function (lambda ()
-                      (require 'emacspeak-new-ediff ))))     
+                      (require 'emacspeak-ediff ))))     
 ;; else older ediff 
 (add-hook 'ediff-load-hooks
-          (function (lambda ()(require 'emacspeak-ediff )))))
+          (function (lambda ()(require 'emacspeak-old-ediff )))))
 
 ;;}}}
 ;;{{{  bbdb 
@@ -785,7 +826,22 @@ the block. Use in conjunction with emacspeak-hide-or-expose-block"
                (emacspeak-auditory-icon 'alarm )))))
 
 ;;}}}
+;;{{{  finder
+(add-hook 'after-load-alist
+          '("finder"
+            (progn
+              (load-library "emacspeak-finder")
+              (unless (file-newer-than-file-p
+                       (concat emacspeak-dir "/"
+                               "emacspeak-finder-inf.el")
+                       (concat emacspeak-dir "/" "emacspeak.el"))
+                (emacspeak-finder-compile-keywords))
+              (load-library "emacspeak-finder-inf")
+              (push 
+               (cons 'emacspeak "Audio Desktop")
+               finder-known-keywords))))
 
+;;}}}
 ;;}}}
 ;;{{{  Submit bugs
 
