@@ -1,4 +1,5 @@
-;;; $Id: lazy-voice-lock.el,v 7.0 1997/11/13 15:32:30 raman Exp $
+;;; lazy-voice-lock.el --- Support mode for voice locking  parts of a buffer
+;;; $Id: lazy-voice-lock.el,v 8.0 1998/05/01 14:15:46 raman Exp $
 ;;; $Author: raman $ 
 ;;; Description:  lazy Voice lock mode for Emacspeak
 ;;{{{  LCD Archive entry: 
@@ -6,15 +7,15 @@
 ;;; LCD Archive Entry:
 ;;; emacspeak| T. V. Raman |raman@adobe.com 
 ;;; A speech interface to Emacs |
-;;; $Date: 1997/11/13 15:32:30 $ |
-;;;  $Revision: 7.0 $ | 
+;;; $Date: 1998/05/01 14:15:46 $ |
+;;;  $Revision: 8.0 $ | 
 ;;; Location undetermined
 ;;;
 
 ;;}}}
 ;;{{{  Copyright:
 
-;;;Copyright (C) 1995, 1996, 1997  T. V. Raman  Adobe Systems Incorporated
+;;;Copyright (C) 1995, 1996, 1997, 1998   T. V. Raman  Adobe Systems Incorporated
 ;;; Copyright (c) 1994, 1995 by Digital Equipment Corporation.
 ;;; All Rights Reserved. 
 ;;;
@@ -276,10 +277,11 @@
 
 
 ;; Make sure lazy-voice-lock.el is supported.
-(if (if (save-match-data (string-match "Lucid\\|XEmacs" (emacs-version)))
-	t
-      (and (= emacs-major-version 19) (< emacs-minor-version 30)))
-    (error "`lazy-voice-lock' was written for Emacs 19.30 or later"))
+;;commenting out  
+;(if (if (save-match-data (string-match "Lucid\\|XEmacs" (emacs-version)))
+;	t
+;      (and (= emacs-major-version 19) (< emacs-minor-version 30)))
+;    (error "`lazy-voice-lock' was written for Emacs 19.30 or later"))
 
 ;; Flush out those lusers who didn't read all of the Commentary.
 ;(if (or (memq 'turn-on-defer-lock voice-lock-mode-hook)
